from .common import InfoExtractor
from ..utils import (
    float_or_none,
    parse_resolution,
    traverse_obj,
    urlencode_postdata,
    variadic,
)


class TubeTuGrazBaseIE(InfoExtractor):
    _NETRC_MACHINE = 'tubetugraz'

    _API_EPISODE = 'https://tube.tugraz.at/search/episode.json'
    _FORMAT_TYPES = ('presentation', 'presenter')

    def _perform_login(self, username, password):
        urlh = self._request_webpage(
            'https://tube.tugraz.at/Shibboleth.sso/Login?target=/paella/ui/index.html',
            None, fatal=False, note='downloading login page', errnote='unable to fetch login page')
        if not urlh:
            return

        response = self._download_webpage_handle(
            urlh.url, None, fatal=False, headers={'referer': urlh.url},
            note='logging in', errnote='unable to log in',
            data=urlencode_postdata({
                'lang': 'de',
                '_eventId_proceed': '',
                'j_username': username,
                'j_password': password,
            }))
        if not response:
            return

        content, urlh = response
        if urlh.url == 'https://tube.tugraz.at/paella/ui/index.html':
            return

        if not self._html_search_regex(
                r'<p\b[^>]*>(Bitte geben Sie einen OTP-Wert ein:)</p>',
                content, 'TFA prompt', default=None):
            self.report_warning('unable to login: incorrect password')
            return

        urlh = self._request_webpage(
            urlh.url, None, fatal=False, headers={'referer': urlh.url},
            note='logging in with TFA', errnote='unable to log in with TFA',
            data=urlencode_postdata({
                'lang': 'de',
                '_eventId_proceed': '',
                'j_tokenNumber': self._get_tfa_info(),
            }))
        if not urlh or urlh.url == 'https://tube.tugraz.at/paella/ui/index.html':
            return

        self.report_warning('unable to login: incorrect TFA code')

    def _extract_episode(self, episode_info):
        video_id = episode_info.get('id')
        formats = list(self._extract_formats(
            traverse_obj(episode_info, ('mediapackage', 'media', 'track')), video_id))

        title = traverse_obj(episode_info, ('mediapackage', 'title'), 'dcTitle')
        series_title = traverse_obj(episode_info, ('mediapackage', 'seriestitle'))
        creator = ', '.join(variadic(traverse_obj(
            episode_info, ('mediapackage', 'creators', 'creator'), 'dcCreator', default='')))
        return {
            'id': video_id,
            'title': title,
            'creator': creator or None,
            'duration': traverse_obj(episode_info, ('mediapackage', 'duration'), 'dcExtent'),
            'series': series_title,
            'series_id': traverse_obj(episode_info, ('mediapackage', 'series'), 'dcIsPartOf'),
            'episode': series_title and title,
            'formats': formats,
        }

    def _set_format_type(self, formats, fmt_type):
        for f in formats:
            f['format_note'] = fmt_type
            if not fmt_type.startswith(self._FORMAT_TYPES[0]):
                f['preference'] = -2
        return formats

    def _extract_formats(self, format_list, video_id):
        has_hls, has_dash = False, False

        for format_info in format_list or []:
            url = traverse_obj(format_info, ('tags', 'url'), 'url')
            if url is None:
                continue

            fmt_type = format_info.get('type') or 'unknown'
            transport = (format_info.get('transport') or 'https').lower()

            if transport == 'https':
                formats = [{
                    'url': url,
                    'abr': float_or_none(traverse_obj(format_info, ('audio', 'bitrate')), 1000),
                    'vbr': float_or_none(traverse_obj(format_info, ('video', 'bitrate')), 1000),
                    'fps': traverse_obj(format_info, ('video', 'framerate')),
                    **parse_resolution(traverse_obj(format_info, ('video', 'resolution'))),
                }]
            elif transport == 'hls':
                has_hls, formats = True, self._extract_m3u8_formats(
                    url, video_id, 'mp4', fatal=False, note=f'downloading {fmt_type} HLS manifest')
            elif transport == 'dash':
                has_dash, formats = True, self._extract_mpd_formats(
                    url, video_id, fatal=False, note=f'downloading {fmt_type} DASH manifest')
            else:
                # RTMP, HDS, SMOOTH, and unknown formats
                # - RTMP url fails on every tested entry until now
                # - HDS url 404's on every tested entry until now
                # - SMOOTH url 404's on every tested entry until now
                continue

            yield from self._set_format_type(formats, fmt_type)

        # TODO: Add test for these
        for fmt_type in self._FORMAT_TYPES:
            if not has_hls:
                hls_formats = self._extract_m3u8_formats(
                    f'https://wowza.tugraz.at/matterhorn_engage/smil:engage-player_{video_id}_{fmt_type}.smil/playlist.m3u8',
                    video_id, 'mp4', fatal=False, note=f'Downloading {fmt_type} HLS manifest', errnote=False) or []
                yield from self._set_format_type(hls_formats, fmt_type)

            if not has_dash:
                dash_formats = self._extract_mpd_formats(
                    f'https://wowza.tugraz.at/matterhorn_engage/smil:engage-player_{video_id}_{fmt_type}.smil/manifest_mpm4sav_mvlist.mpd',
                    video_id, fatal=False, note=f'Downloading {fmt_type} DASH manifest', errnote=False)
                yield from self._set_format_type(dash_formats, fmt_type)


class TubeTuGrazIE(TubeTuGrazBaseIE):
    IE_DESC = 'tube.tugraz.at'

    _VALID_URL = r'''(?x)
        https?://tube\.tugraz\.at/(?:
            paella/ui/watch\.html\?(?:[^#]*&)?id=|
            portal/watch/
        )(?P<id>[0-9a-fA-F]{8}-(?:[0-9a-fA-F]{4}-){3}[0-9a-fA-F]{12})
    '''
    _TESTS = [
        {
            'url': 'https://tube.tugraz.at/paella/ui/watch.html?id=f2634392-e40e-4ac7-9ddc-47764aa23d40',
            'md5': 'a23a3d5c9aaca2b84932fdba66e17145',
            'info_dict': {
                'id': 'f2634392-e40e-4ac7-9ddc-47764aa23d40',
                'ext': 'mp4',
                'title': '#6 (23.11.2017)',
                'episode': '#6 (23.11.2017)',
                'series': '[INB03001UF] Einführung in die strukturierte Programmierung',
                'duration': 3295818,
                'series_id': 'b1192fff-2aa7-4bf0-a5cf-7b15c3bd3b34',
                'creators': ['Safran C'],
            },
        }, {
            'url': 'https://tube.tugraz.at/paella/ui/watch.html?id=2df6d787-e56a-428d-8ef4-d57f07eef238',
            'md5': 'de0d854a56bf7318d2b693fe1adb89a5',
            'info_dict': {
                'id': '2df6d787-e56a-428d-8ef4-d57f07eef238',
                'title': 'TubeTuGraz video #2df6d787-e56a-428d-8ef4-d57f07eef238',
                'ext': 'mp4',
            },
            'expected_warnings': ['Extractor failed to obtain "title"'],
        }, {
            # Portal URL format
            'url': 'https://tube.tugraz.at/portal/watch/ab28ec60-8cbe-4f1a-9b96-a95add56c612',
            'only_matching': True,
        },
    ]

    def _real_extract(self, url):
        video_id = self._match_id(url)
        episode_data = self._download_json(
            self._API_EPISODE, video_id, query={'id': video_id, 'limit': 1}, note='Downloading episode metadata')

        episode_info = traverse_obj(episode_data, ('search-results', 'result'), default={'id': video_id})
        return self._extract_episode(episode_info)


class TubeTuGrazSeriesIE(TubeTuGrazBaseIE):
    _VALID_URL = r'''(?x)
        https?://tube\.tugraz\.at/paella/ui/browse\.html\?series=
        (?P<id>[0-9a-fA-F]{8}-(?:[0-9a-fA-F]{4}-){3}[0-9a-fA-F]{12})
    '''
    _TESTS = [{
        'url': 'https://tube.tugraz.at/paella/ui/browse.html?series=0e6351b7-c372-491e-8a49-2c9b7e21c5a6',
        'id': '0e6351b7-c372-491e-8a49-2c9b7e21c5a6',
        'info_dict': {
            'id': '0e6351b7-c372-491e-8a49-2c9b7e21c5a6',
            'title': '[209351] Strassenwesen',
        },
        'playlist': [
            {
                'info_dict': {
                    'id': 'ee17ce5d-34e2-48b7-a76a-fed148614e11',
                    'series_id': '0e6351b7-c372-491e-8a49-2c9b7e21c5a6',
                    'ext': 'mp4',
                    'title': '#4 Detailprojekt',
                    'episode': '#4 Detailprojekt',
                    'series': '[209351] Strassenwesen',
                    'creator': 'Neuhold R',
                    'duration': 6127024,
                },
            },
            {
                'info_dict': {
                    'id': '87350498-799a-44d3-863f-d1518a98b114',
                    'series_id': '0e6351b7-c372-491e-8a49-2c9b7e21c5a6',
                    'ext': 'mp4',
                    'title': '#3 Generelles Projekt',
                    'episode': '#3 Generelles Projekt',
                    'series': '[209351] Strassenwesen',
                    'creator': 'Neuhold R',
                    'duration': 5374422,
                },
            },
            {
                'info_dict': {
                    'id': '778599ea-489e-4189-9e05-3b4888e19bcd',
                    'series_id': '0e6351b7-c372-491e-8a49-2c9b7e21c5a6',
                    'ext': 'mp4',
                    'title': '#2 Vorprojekt',
                    'episode': '#2 Vorprojekt',
                    'series': '[209351] Strassenwesen',
                    'creator': 'Neuhold R',
                    'duration': 5566404,
                },
            },
            {
                'info_dict': {
                    'id': '75e4c71c-d99d-4e56-b0e6-4f2bcdf11f29',
                    'series_id': '0e6351b7-c372-491e-8a49-2c9b7e21c5a6',
                    'ext': 'mp4',
                    'title': '#1 Variantenstudium',
                    'episode': '#1 Variantenstudium',
                    'series': '[209351] Strassenwesen',
                    'creator': 'Neuhold R',
                    'duration': 5420200,
                },
            },
        ],
        'playlist_mincount': 4,
    }]

    def _real_extract(self, url):
        playlist_id = self._match_id(url)
        episodes_data = self._download_json(
            self._API_EPISODE, playlist_id, query={'sid': playlist_id}, note='Downloading episode list')
        series_data = self._download_json(
            'https://tube.tugraz.at/series/series.json', playlist_id, fatal=False,
            note='downloading series metadata', errnote='failed to download series metadata',
            query={
                'seriesId': playlist_id,
                'count': 1,
                'sort': 'TITLE',
            })

        return self.playlist_result(
            map(self._extract_episode, episodes_data['search-results']['result']), playlist_id,
            traverse_obj(series_data, ('catalogs', 0, 'http://purl.org/dc/terms/', 'title', 0, 'value')))



import json
import urllib.parse

from .common import InfoExtractor
from ..networking.exceptions import HTTPError
from ..utils import (
    ExtractorError,
    filter_dict,
    float_or_none,
    join_nonempty,
    mimetype2ext,
    parse_iso8601,
    unsmuggle_url,
    update_url_query,
    url_or_none,
)
from ..utils.traversal import traverse_obj


class StreaksBaseIE(InfoExtractor):
    _API_URL_TEMPLATE = 'https://{}.api.streaks.jp/v1/projects/{}/medias/{}{}'
    _GEO_BYPASS = False
    _GEO_COUNTRIES = ['JP']

    def _extract_from_streaks_api(self, project_id, media_id, headers=None, query=None, ssai=False, live_from_start=False):
        try:
            response = self._download_json(
                self._API_URL_TEMPLATE.format('playback', project_id, media_id, ''),
                media_id, 'Downloading STREAKS playback API JSON', headers={
                    'Accept': 'application/json',
                    'Origin': 'https://players.streaks.jp',
                    **self.geo_verification_headers(),
                    **(headers or {}),
                })
        except ExtractorError as e:
            if isinstance(e.cause, HTTPError) and e.cause.status in (403, 404):
                error = self._parse_json(e.cause.response.read().decode(), media_id, fatal=False)
                message = traverse_obj(error, ('message', {str}))
                code = traverse_obj(error, ('code', {str}))
                error_id = traverse_obj(error, ('id', {int}))
                if code == 'REQUEST_FAILED':
                    if error_id == 124:
                        self.raise_geo_restricted(countries=self._GEO_COUNTRIES)
                    elif error_id == 126:
                        raise ExtractorError('Access is denied (possibly due to invalid/missing API key)')
                if code == 'MEDIA_NOT_FOUND':
                    raise ExtractorError(join_nonempty(code, message, delim=': '), expected=True)
                if code or message:
                    raise ExtractorError(join_nonempty(code, error_id, message, delim=': '))
            raise

        streaks_id = response['id']
        live_status = {
            'clip': 'was_live',
            'file': 'not_live',
            'linear': 'is_live',
            'live': 'is_live',
        }.get(response.get('type'))

        formats, subtitles = [], {}
        drm_formats = False

        for source in traverse_obj(response, ('sources', lambda _, v: v['src'])):
            if source.get('key_systems'):
                drm_formats = True
                continue

            src_url = source['src']
            is_live = live_status == 'is_live'
            ext = mimetype2ext(source.get('type'))
            if ext != 'm3u8':
                self.report_warning(f'Unsupported stream type: {ext}')
                continue

            if is_live and ssai:
                session_params = traverse_obj(self._download_json(
                    self._API_URL_TEMPLATE.format('ssai', project_id, streaks_id, '/ssai/session'),
                    media_id, 'Downloading session parameters',
                    headers={'Content-Type': 'application/json', 'Accept': 'application/json'},
                    data=json.dumps({'id': source['id']}).encode(),
                ), (0, 'query', {urllib.parse.parse_qs}))
                src_url = update_url_query(src_url, session_params)

            fmts, subs = self._extract_m3u8_formats_and_subtitles(
                src_url, media_id, 'mp4', m3u8_id='hls', fatal=False, live=is_live, query=query)
            for fmt in fmts:
                if live_from_start:
                    fmt.setdefault('downloader_options', {}).update({'ffmpeg_args': ['-live_start_index', '0']})
                    fmt['is_from_start'] = True
            formats.extend(fmts)
            self._merge_subtitles(subs, target=subtitles)

        if not formats and drm_formats:
            self.report_drm(media_id)
        self._remove_duplicate_formats(formats)

        for subs in traverse_obj(response, (
            'tracks', lambda _, v: v['kind'] in ('captions', 'subtitles') and url_or_none(v['src']),
        )):
            lang = traverse_obj(subs, ('srclang', {str.lower})) or 'ja'
            subtitles.setdefault(lang, []).append({'url': subs['src']})

        return {
            'id': streaks_id,
            'display_id': media_id,
            'formats': formats,
            'live_status': live_status,
            'subtitles': subtitles,
            'uploader_id': project_id,
            **traverse_obj(response, {
                'title': ('name', {str}),
                'description': ('description', {str}, filter),
                'duration': ('duration', {float_or_none}),
                'modified_timestamp': ('updated_at', {parse_iso8601}),
                'tags': ('tags', ..., {str}),
                'thumbnails': (('poster', 'thumbnail'), 'src', {'url': {url_or_none}}),
                'timestamp': ('created_at', {parse_iso8601}),
            }),
        }


class StreaksIE(StreaksBaseIE):
    _VALID_URL = [
        r'https?://players\.streaks\.jp/(?P<project_id>[\w-]+)/[\da-f]+/index\.html\?(?:[^#]+&)?m=(?P<id>(?:ref:)?[\w-]+)',
        r'https?://playback\.api\.streaks\.jp/v1/projects/(?P<project_id>[\w-]+)/medias/(?P<id>(?:ref:)?[\w-]+)',
    ]
    _EMBED_REGEX = [rf'<iframe\s+[^>]*\bsrc\s*=\s*["\'](?P<url>{_VALID_URL[0]})']
    _TESTS = [{
        'url': 'https://players.streaks.jp/tipness/08155cd19dc14c12bebefb69b92eafcc/index.html?m=dbdf2df35b4d483ebaeeaeb38c594647',
        'info_dict': {
            'id': 'dbdf2df35b4d483ebaeeaeb38c594647',
            'ext': 'mp4',
            'title': '3shunenCM_edit.mp4',
            'display_id': 'dbdf2df35b4d483ebaeeaeb38c594647',
            'duration': 47.533,
            'live_status': 'not_live',
            'modified_date': '20230726',
            'modified_timestamp': 1690356180,
            'timestamp': 1690355996,
            'upload_date': '20230726',
            'uploader_id': 'tipness',
        },
    }, {
        'url': 'https://players.streaks.jp/ktv-web/0298e8964c164ab384c07ef6e08c444b/index.html?m=ref:mycoffeetime_250317',
        'info_dict': {
            'id': 'dccdc079e3fd41f88b0c8435e2d453ab',
            'ext': 'mp4',
            'title': 'わたしの珈琲時間_250317',
            'display_id': 'ref:mycoffeetime_250317',
            'duration': 122.99,
            'live_status': 'not_live',
            'modified_date': '20250310',
            'modified_timestamp': 1741586302,
            'thumbnail': r're:https?://.+\.jpg',
            'timestamp': 1741585839,
            'upload_date': '20250310',
            'uploader_id': 'ktv-web',
        },
    }, {
        'url': 'https://playback.api.streaks.jp/v1/projects/ktv-web/medias/b5411938e1e5435dac71edf829dd4813',
        'info_dict': {
            'id': 'b5411938e1e5435dac71edf829dd4813',
            'ext': 'mp4',
            'title': 'KANTELE_SYUSEi_0630',
            'display_id': 'b5411938e1e5435dac71edf829dd4813',
            'live_status': 'not_live',
            'modified_date': '20250122',
            'modified_timestamp': 1737522999,
            'thumbnail': r're:https?://.+\.jpg',
            'timestamp': 1735205137,
            'upload_date': '20241226',
            'uploader_id': 'ktv-web',
        },
    }, {
        # TVer Olympics: website already down, but api remains accessible
        'url': 'https://playback.api.streaks.jp/v1/projects/tver-olympic/medias/ref:sp_240806_1748_dvr',
        'info_dict': {
            'id': 'c10f7345adb648cf804d7578ab93b2e3',
            'ext': 'mp4',
            'title': 'サッカー 男子 準決勝_dvr',
            'display_id': 'ref:sp_240806_1748_dvr',
            'duration': 12960.0,
            'live_status': 'was_live',
            'modified_date': '20240805',
            'modified_timestamp': 1722896263,
            'timestamp': 1722777618,
            'upload_date': '20240804',
            'uploader_id': 'tver-olympic',
        },
    }, {
        # TBS FREE: 24-hour stream
        'url': 'https://playback.api.streaks.jp/v1/projects/tbs/medias/ref:simul-02',
        'info_dict': {
            'id': 'c4e83a7b48f4409a96adacec674b4e22',
            'ext': 'mp4',
            'title': str,
            'display_id': 'ref:simul-02',
            'live_status': 'is_live',
            'modified_date': '20241031',
            'modified_timestamp': 1730339858,
            'timestamp': 1705466840,
            'upload_date': '20240117',
            'uploader_id': 'tbs',
        },
    }, {
        # DRM protected
        'url': 'https://players.streaks.jp/sp-jbc/a12d7ee0f40c49d6a0a2bff520639677/index.html?m=5f89c62f37ee4a68be8e6e3b1396c7d8',
        'only_matching': True,
    }]
    _WEBPAGE_TESTS = [{
        'url': 'https://event.play.jp/playnext2023/',
        'info_dict': {
            'id': '2d975178293140dc8074a7fc536a7604',
            'ext': 'mp4',
            'title': 'PLAY NEXTキームービー（本番）',
            'uploader_id': 'play',
            'duration': 17.05,
            'thumbnail': r're:https?://.+\.jpg',
            'timestamp': 1668387517,
            'upload_date': '20221114',
            'modified_timestamp': 1739411523,
            'modified_date': '20250213',
            'live_status': 'not_live',
        },
    }, {
        'url': 'https://wowshop.jp/Page/special/cooking_goods/?bid=wowshop&srsltid=AfmBOor_phUNoPEE_UCPiGGSCMrJE5T2US397smvsbrSdLqUxwON0el4',
        'playlist_mincount': 2,
        'info_dict': {
            'id': '?bid=wowshop&srsltid=AfmBOor_phUNoPEE_UCPiGGSCMrJE5T2US397smvsbrSdLqUxwON0el4',
            'title': 'ワンランク上の料理道具でとびきりの“おいしい”を食卓へ｜wowshop',
            'description': 'md5:914b5cb8624fc69274c7fb7b2342958f',
            'age_limit': 0,
            'thumbnail': 'https://wowshop.jp/Page/special/cooking_goods/images/ogp.jpg',
        },
    }]

    def _real_extract(self, url):
        url, smuggled_data = unsmuggle_url(url, {})
        project_id, media_id = self._match_valid_url(url).group('project_id', 'id')

        return self._extract_from_streaks_api(
            project_id, media_id, headers=filter_dict({
                'X-Streaks-Api-Key': smuggled_data.get('api_key'),
            }))



import re

from .common import InfoExtractor
from ..utils import determine_ext, js_to_json, mimetype2ext, traverse_obj


class TV24UAVideoIE(InfoExtractor):
    _VALID_URL = r'https?://24tv\.ua/news/showPlayer\.do.*?(?:\?|&)objectId=(?P<id>\d+)'
    _EMBED_REGEX = [rf'<iframe[^>]+?src=["\']?(?P<url>{_VALID_URL})["\']?']
    IE_NAME = '24tv.ua'
    _TESTS = [{
        'url': 'https://24tv.ua/news/showPlayer.do?objectId=2074790&videoUrl=2022/07/2074790&w=640&h=360',
        'info_dict': {
            'id': '2074790',
            'ext': 'mp4',
            'title': 'У Харкові ворожа ракета прилетіла в будинок, де слухали пісні про "офіцерів-росіян"',
            'thumbnail': r're:^https?://.*\.jpe?g',
        },
    }, {
        'url': 'https://24tv.ua/news/showPlayer.do?videoUrl=2022/07/2074790&objectId=2074790&w=640&h=360',
        'only_matching': True,
    }]

    _WEBPAGE_TESTS = [
        {
            # iframe embed created from share menu.
            'url': 'data:text/html,%3Ciframe%20src=%22https://24tv.ua/news/showPlayer.do?objectId=1886193&videoUrl'
                   '=2022/03/1886193&w=640&h=360%22%20width=%22640%22%20height=%22360%22%20frameborder=%220%22'
                   '%20scrolling=%22no%22%3E%3C/iframe%3E',
            'info_dict': {
                'id': '1886193',
                'ext': 'mp4',
                'title': 'Росіяни руйнують Бородянку на Київщині та стріляють з літаків по мешканцях: шокуючі фото',
                'thumbnail': r're:^https?://.*\.jpe?g',
            },
        },
        {
            'url': 'https://24tv.ua/vipalyuyut-nashi-mista-sela-dsns-pokazali-motoroshni-naslidki_n1883966',
            'info_dict': {
                'id': '1883966',
                'ext': 'mp4',
                'title': 'Випалюють наші міста та села, – моторошні наслідки обстрілів на Чернігівщині',
                'thumbnail': r're:^https?://.*\.jpe?g',
            },
            'params': {'allowed_extractors': ['Generic', '24tv.ua']},
        },
    ]

    def _real_extract(self, url):
        video_id = self._match_id(url)
        webpage = self._download_webpage(url, video_id)

        formats = []
        subtitles = {}
        for j in re.findall(r'vPlayConfig\.sources\s*=\s*(?P<json>\[{\s*(?s:.+?)\s*}])', webpage):
            sources = self._parse_json(j, video_id, fatal=False, ignore_extra=True, transform_source=js_to_json, errnote='') or []
            for source in sources:
                if mimetype2ext(traverse_obj(source, 'type')) == 'm3u8':
                    f, s = self._extract_m3u8_formats_and_subtitles(source['src'], video_id)
                    formats.extend(f)
                    self._merge_subtitles(subtitles, s)
                else:
                    formats.append({
                        'url': source['src'],
                        'ext': determine_ext(source['src']),
                    })
        thumbnail = traverse_obj(
            self._search_json(
                r'var\s*vPlayConfig\s*=\s*', webpage, 'thumbnail',
                video_id, default=None, transform_source=js_to_json), 'poster')
        return {
            'id': video_id,
            'formats': formats,
            'subtitles': subtitles,
            'thumbnail': thumbnail or self._og_search_thumbnail(webpage),
            'title': self._generic_title('', webpage),
            'description': self._og_search_description(webpage, default=None),
        }



import re

from .common import InfoExtractor
from ..utils import (
    float_or_none,
    get_element_by_class,
    get_element_by_id,
    unified_strdate,
)


class FreesoundIE(InfoExtractor):
    _VALID_URL = r'https?://(?:www\.)?freesound\.org/people/[^/]+/sounds/(?P<id>[^/]+)'
    _TEST = {
        'url': 'http://www.freesound.org/people/miklovan/sounds/194503/',
        'md5': '12280ceb42c81f19a515c745eae07650',
        'info_dict': {
            'id': '194503',
            'ext': 'mp3',
            'title': 'gulls in the city.wav',
            'description': 'the sounds of seagulls in the city',
            'duration': 130.233,
            'uploader': 'miklovan',
            'upload_date': '20130715',
            'tags': list,
        },
    }

    def _real_extract(self, url):
        audio_id = self._match_id(url)

        webpage = self._download_webpage(url, audio_id)

        audio_url = self._og_search_property('audio', webpage, 'song url')
        title = self._og_search_property('audio:title', webpage, 'song title')

        description = self._html_search_regex(
            r'(?s)id=["\']sound_description["\'][^>]*>(.+?)</div>',
            webpage, 'description', fatal=False)

        duration = float_or_none(
            get_element_by_class('duration', webpage), scale=1000)

        upload_date = unified_strdate(get_element_by_id('sound_date', webpage))
        uploader = self._og_search_property(
            'audio:artist', webpage, 'uploader', fatal=False)

        channels = self._html_search_regex(
            r'Channels</dt><dd>(.+?)</dd>', webpage,
            'channels info', fatal=False)

        tags_str = get_element_by_class('tags', webpage)
        tags = re.findall(r'<a[^>]+>([^<]+)', tags_str) if tags_str else None

        audio_url = re.sub(r'^https?://freesound\.org(https?://)', r'\1', audio_url)
        audio_urls = [audio_url]

        LQ_FORMAT = '-lq.mp3'
        if LQ_FORMAT in audio_url:
            audio_urls.append(audio_url.replace(LQ_FORMAT, '-hq.mp3'))

        formats = [{
            'url': format_url,
            'format_note': channels,
            'quality': quality,
        } for quality, format_url in enumerate(audio_urls)]

        return {
            'id': audio_id,
            'title': title,
            'description': description,
            'duration': duration,
            'uploader': uploader,
            'upload_date': upload_date,
            'tags': tags,
            'formats': formats,
        }



from .common import InfoExtractor
from ..utils import (
    int_or_none,
    str_or_none,
    try_get,
    unified_timestamp,
    update_url_query,
    urljoin,
)


class CPACIE(InfoExtractor):
    IE_NAME = 'cpac'
    _VALID_URL = r'https?://(?:www\.)?cpac\.ca/(?P<fr>l-)?episode\?id=(?P<id>[\da-f]{8}(?:-[\da-f]{4}){3}-[\da-f]{12})'
    _TEST = {
        # 'url': 'http://www.cpac.ca/en/programs/primetime-politics/episodes/65490909',
        'url': 'https://www.cpac.ca/episode?id=fc7edcae-4660-47e1-ba61-5b7f29a9db0f',
        'md5': 'e46ad699caafd7aa6024279f2614e8fa',
        'info_dict': {
            'id': 'fc7edcae-4660-47e1-ba61-5b7f29a9db0f',
            'ext': 'mp4',
            'upload_date': '20220215',
            'title': 'News Conference to Celebrate National Kindness Week – February 15, 2022',
            'description': 'md5:466a206abd21f3a6f776cdef290c23fb',
            'timestamp': 1644901200,
        },
        'params': {
            'format': 'bestvideo',
            'hls_prefer_native': True,
        },
    }

    def _real_extract(self, url):
        video_id = self._match_id(url)
        url_lang = 'fr' if '/l-episode?' in url else 'en'

        content = self._download_json(
            'https://www.cpac.ca/api/1/services/contentModel.json?url=/site/website/episode/index.xml&crafterSite=cpacca&id=' + video_id,
            video_id)
        video_url = try_get(content, lambda x: x['page']['details']['videoUrl'], str)
        formats = []
        if video_url:
            content = content['page']
            title = str_or_none(content['details'][f'title_{url_lang}_t'])
            formats = self._extract_m3u8_formats(video_url, video_id, m3u8_id='hls', ext='mp4')
            for fmt in formats:
                # prefer language to match URL
                fmt_lang = fmt.get('language')
                if fmt_lang == url_lang:
                    fmt['language_preference'] = 10
                elif not fmt_lang:
                    fmt['language_preference'] = -1
                else:
                    fmt['language_preference'] = -10

        category = str_or_none(content['details'][f'category_{url_lang}_t'])

        def is_live(v_type):
            return (v_type == 'live') if v_type is not None else None

        return {
            'id': video_id,
            'formats': formats,
            'title': title,
            'description': str_or_none(content['details'].get(f'description_{url_lang}_t')),
            'timestamp': unified_timestamp(content['details'].get('liveDateTime')),
            'categories': [category] if category else None,
            'thumbnail': urljoin(url, str_or_none(content['details'].get(f'image_{url_lang}_s'))),
            'is_live': is_live(content['details'].get('type')),
        }


class CPACPlaylistIE(InfoExtractor):
    IE_NAME = 'cpac:playlist'
    _VALID_URL = r'(?i)https?://(?:www\.)?cpac\.ca/(?:program|search|(?P<fr>emission|rechercher))\?(?:[^&]+&)*?(?P<id>(?:id=\d+|programId=\d+|key=[^&]+))'

    _TESTS = [{
        'url': 'https://www.cpac.ca/program?id=6',
        'info_dict': {
            'id': 'id=6',
            'title': 'Headline Politics',
            'description': 'Watch CPAC’s signature long-form coverage of the day’s pressing political events as they unfold.',
        },
        'playlist_count': 10,
    }, {
        'url': 'https://www.cpac.ca/search?key=hudson&type=all&order=desc',
        'info_dict': {
            'id': 'key=hudson',
            'title': 'hudson',
        },
        'playlist_count': 22,
    }, {
        'url': 'https://www.cpac.ca/search?programId=50',
        'info_dict': {
            'id': 'programId=50',
            'title': '50',
        },
        'playlist_count': 9,
    }, {
        'url': 'https://www.cpac.ca/emission?id=6',
        'only_matching': True,
    }, {
        'url': 'https://www.cpac.ca/rechercher?key=hudson&type=all&order=desc',
        'only_matching': True,
    }]

    def _real_extract(self, url):
        video_id = self._match_id(url)
        url_lang = 'fr' if any(x in url for x in ('/emission?', '/rechercher?')) else 'en'
        pl_type, list_type = ('program', 'itemList') if any(x in url for x in ('/program?', '/emission?')) else ('search', 'searchResult')
        api_url = (
            f'https://www.cpac.ca/api/1/services/contentModel.json?url=/site/website/{pl_type}/index.xml&crafterSite=cpacca&{video_id}')
        content = self._download_json(api_url, video_id)
        entries = []
        total_pages = int_or_none(try_get(content, lambda x: x['page'][list_type]['totalPages']), default=1)
        for page in range(1, total_pages + 1):
            if page > 1:
                api_url = update_url_query(api_url, {'page': page})
                content = self._download_json(
                    api_url, video_id,
                    note=f'Downloading continuation - {page}',
                    fatal=False)

            for item in try_get(content, lambda x: x['page'][list_type]['item'], list) or []:
                episode_url = urljoin(url, try_get(item, lambda x: x[f'url_{url_lang}_s']))
                if episode_url:
                    entries.append(episode_url)

        return self.playlist_result(
            (self.url_result(entry) for entry in entries),
            playlist_id=video_id,
            playlist_title=try_get(content, lambda x: x['page']['program'][f'title_{url_lang}_t']) or video_id.split('=')[-1],
            playlist_description=try_get(content, lambda x: x['page']['program'][f'description_{url_lang}_t']),
        )



import json

from .common import InfoExtractor
from ..networking.exceptions import HTTPError
from ..utils import (
    ExtractorError,
    GeoRestrictedError,
    float_or_none,
    traverse_obj,
    try_call,
)


class OnDemandChinaEpisodeIE(InfoExtractor):
    _VALID_URL = r'https?://www\.ondemandchina\.com/\w+/watch/(?P<series>[\w-]+)/(?P<id>ep-(?P<ep>\d+))'
    _TESTS = [{
        'url': 'https://www.ondemandchina.com/en/watch/together-against-covid-19/ep-1',
        'info_dict': {
            'id': '264394',
            'ext': 'mp4',
            'duration': 3256.88,
            'title': 'EP 1 The Calling',
            'alt_title': '第1集 令出如山',
            'thumbnail': 'https://d2y2efdi5wgkcl.cloudfront.net/fit-in/256x256/media-io/2020/9/11/image.d9816e81.jpg',
            'description': '疫情严峻，党政军民学、东西南北中协同应考',
            'tags': ['Social Humanities', 'Documentary', 'Medical', 'Social'],
        },
    }]

    _QUERY = '''
        query Episode($programSlug: String!, $episodeNumber: Int!) {
            episode(
                programSlug: $programSlug
                episodeNumber: $episodeNumber
                kind: "series"
                part: null
            ) {
                id
                title
                titleEn
                titleKo
                titleZhHans
                titleZhHant
                synopsis
                synopsisEn
                synopsisKo
                synopsisZhHans
                synopsisZhHant
                videoDuration
                images {
                    thumbnail
                }
            }
        }'''

    def _real_extract(self, url):
        program_slug, display_id, ep_number = self._match_valid_url(url).group('series', 'id', 'ep')
        webpage = self._download_webpage(url, display_id)

        video_info = self._download_json(
            'https://odc-graphql.odkmedia.io/graphql', display_id,
            headers={'Content-type': 'application/json'},
            data=json.dumps({
                'operationName': 'Episode',
                'query': self._QUERY,
                'variables': {
                    'programSlug': program_slug,
                    'episodeNumber': int(ep_number),
                },
            }).encode())['data']['episode']

        try:
            source_json = self._download_json(
                f'https://odkmedia.io/odc/api/v2/playback/{video_info["id"]}/', display_id,
                headers={'Authorization': '', 'service-name': 'odc'})
        except ExtractorError as e:
            if isinstance(e.cause, HTTPError):
                error_data = self._parse_json(e.cause.response.read(), display_id)['detail']
                raise GeoRestrictedError(error_data)

        formats, subtitles = [], {}
        for source in traverse_obj(source_json, ('sources', ...)):
            if source.get('type') == 'hls':
                fmts, subs = self._extract_m3u8_formats_and_subtitles(source.get('url'), display_id)
                formats.extend(fmts)
                self._merge_subtitles(subs, target=subtitles)
            else:
                self.report_warning(f'Unsupported format {source.get("type")}', display_id)

        return {
            'id': str(video_info['id']),
            'duration': float_or_none(video_info.get('videoDuration'), 1000),
            'thumbnail': (traverse_obj(video_info, ('images', 'thumbnail'))
                          or self._html_search_meta(['og:image', 'twitter:image'], webpage)),
            'title': (traverse_obj(video_info, 'title', 'titleEn')
                      or self._html_search_meta(['og:title', 'twitter:title'], webpage)
                      or self._html_extract_title(webpage)),
            'alt_title': traverse_obj(video_info, 'titleKo', 'titleZhHans', 'titleZhHant'),
            'description': (traverse_obj(
                video_info, 'synopsisEn', 'synopsisKo', 'synopsisZhHans', 'synopsisZhHant', 'synopisis')
                or self._html_search_meta(['og:description', 'twitter:description', 'description'], webpage)),
            'formats': formats,
            'subtitles': subtitles,
            'tags': try_call(lambda: self._html_search_meta('keywords', webpage).split(', ')),
        }



def what(file=None, h=None):
    """Detect format of image (Currently supports jpeg, png, webp, gif only)
    Ref: https://github.com/python/cpython/blob/3.11/Lib/imghdr.py
    Ref: https://www.w3.org/Graphics/JPEG/itu-t81.pdf
    """
    if h is None:
        with open(file, 'rb') as f:
            h = f.read(12)

    if h.startswith(b'RIFF') and h.startswith(b'WEBP', 8):
        return 'webp'

    if h.startswith(b'\x89PNG'):
        return 'png'

    if h.startswith(b'\xFF\xD8\xFF'):
        return 'jpeg'

    if h.startswith(b'GIF'):
        return 'gif'

    return None



import hashlib
import json
import re
import time

from .common import InfoExtractor
from ..networking.exceptions import HTTPError
from ..utils import ExtractorError, int_or_none, join_nonempty, url_or_none
from ..utils.traversal import traverse_obj


class DangalPlayBaseIE(InfoExtractor):
    _NETRC_MACHINE = 'dangalplay'
    _REGION = 'IN'
    _OTV_USER_ID = None
    _LOGIN_HINT = (
        'Pass credentials as -u "token" -p "USER_ID" '
        '(where USER_ID is the value of "otv_user_id" in your browser local storage). '
        'Your login region can be optionally suffixed to the username as @REGION '
        '(where REGION is the two-letter "region" code found in your browser local storage), '
        'e.g.: -u "token@IN" -p "USER_ID"')
    _API_BASE = 'https://ottapi.dangalplay.com'
    _AUTH_TOKEN = 'jqeGWxRKK7FK5zEk3xCM'  # from https://www.dangalplay.com/main.48ad19e24eb46acccef3.js
    _SECRET_KEY = 'f53d31a4377e4ef31fa0'  # same as above

    def _perform_login(self, username, password):
        if self._OTV_USER_ID:
            return
        mobj = re.fullmatch(r'token(?:@(?P<region>[A-Z]{2}))?', username)
        if not mobj or not re.fullmatch(r'[\da-f]{32}', password):
            raise ExtractorError(self._LOGIN_HINT, expected=True)
        if region := mobj.group('region'):
            self._REGION = region
        self.write_debug(f'Setting login region to "{self._REGION}"')
        self._OTV_USER_ID = password

    def _real_initialize(self):
        if not self._OTV_USER_ID:
            self.raise_login_required(f'Login required. {self._LOGIN_HINT}', method=None)

    def _extract_episode_info(self, metadata, episode_slug, series_slug):
        return {
            'display_id': episode_slug,
            'episode_number': int_or_none(self._search_regex(
                r'ep-(?:number-)?(\d+)', episode_slug, 'episode number', default=None)),
            'season_number': int_or_none(self._search_regex(
                r'season-(\d+)', series_slug, 'season number', default='1')),
            'series': series_slug,
            **traverse_obj(metadata, {
                'id': ('content_id', {str}),
                'title': ('display_title', {str}),
                'episode': ('title', {str}),
                'series': ('show_name', {str}, filter),
                'series_id': ('catalog_id', {str}),
                'duration': ('duration', {int_or_none}),
                'release_timestamp': ('release_date_uts', {int_or_none}),
            }),
        }

    def _call_api(self, path, display_id, note='Downloading JSON metadata', fatal=True, query={}):
        return self._download_json(
            f'{self._API_BASE}/{path}', display_id, note, fatal=fatal,
            headers={'Accept': 'application/json'}, query={
                'auth_token': self._AUTH_TOKEN,
                'region': self._REGION,
                **query,
            })


class DangalPlayIE(DangalPlayBaseIE):
    IE_NAME = 'dangalplay'
    _VALID_URL = r'https?://(?:www\.)?dangalplay.com/shows/(?P<series>[^/?#]+)/(?P<id>(?!episodes)[^/?#]+)/?(?:$|[?#])'
    _TESTS = [{
        'url': 'https://www.dangalplay.com/shows/kitani-mohabbat-hai-season-2/kitani-mohabbat-hai-season-2-ep-number-01',
        'info_dict': {
            'id': '647c61dc1e7171310dcd49b4',
            'ext': 'mp4',
            'release_timestamp': 1262304000,
            'episode_number': 1,
            'episode': 'EP 1 | KITANI MOHABBAT HAI SEASON 2',
            'series': 'kitani-mohabbat-hai-season-2',
            'season_number': 2,
            'title': 'EP 1 | KITANI MOHABBAT HAI SEASON 2',
            'release_date': '20100101',
            'duration': 2325,
            'season': 'Season 2',
            'display_id': 'kitani-mohabbat-hai-season-2-ep-number-01',
            'series_id': '645c9ea41e717158ca574966',
        },
    }, {
        'url': 'https://www.dangalplay.com/shows/milke-bhi-hum-na-mile/milke-bhi-hum-na-mile-ep-number-01',
        'info_dict': {
            'id': '65d31d9ba73b9c3abd14a7f3',
            'ext': 'mp4',
            'episode': 'EP 1 | MILKE BHI HUM NA MILE',
            'release_timestamp': 1708367411,
            'episode_number': 1,
            'season': 'Season 1',
            'title': 'EP 1 | MILKE BHI HUM NA MILE',
            'duration': 156048,
            'release_date': '20240219',
            'season_number': 1,
            'series': 'MILKE BHI HUM NA MILE',
            'series_id': '645c9ea41e717158ca574966',
            'display_id': 'milke-bhi-hum-na-mile-ep-number-01',
        },
    }]

    def _generate_api_data(self, data):
        catalog_id = data['catalog_id']
        content_id = data['content_id']
        timestamp = str(int(time.time()))
        unhashed = ''.join((catalog_id, content_id, self._OTV_USER_ID, timestamp, self._SECRET_KEY))

        return json.dumps({
            'catalog_id': catalog_id,
            'content_id': content_id,
            'category': '',
            'region': self._REGION,
            'auth_token': self._AUTH_TOKEN,
            'id': self._OTV_USER_ID,
            'md5': hashlib.md5(unhashed.encode()).hexdigest(),
            'ts': timestamp,
        }, separators=(',', ':')).encode()

    def _real_extract(self, url):
        series_slug, episode_slug = self._match_valid_url(url).group('series', 'id')
        metadata = self._call_api(
            f'catalogs/shows/{series_slug}/episodes/{episode_slug}.gzip',
            episode_slug, query={'item_language': ''})['data']

        try:
            details = self._download_json(
                f'{self._API_BASE}/v2/users/get_all_details.gzip', episode_slug,
                'Downloading playback details JSON', headers={
                    'Accept': 'application/json',
                    'Content-Type': 'application/json',
                }, data=self._generate_api_data(metadata))['data']
        except ExtractorError as e:
            if isinstance(e.cause, HTTPError) and e.cause.status == 422:
                error_info = traverse_obj(e.cause.response.read().decode(), ({json.loads}, 'error', {dict})) or {}
                error_code = error_info.get('code')
                if error_code == '1016':
                    self.raise_login_required(
                        f'Your token has expired or is invalid. {self._LOGIN_HINT}', method=None)
                elif error_code == '4028':
                    self.raise_login_required(
                        f'Your login region is unspecified or incorrect. {self._LOGIN_HINT}', method=None)
                raise ExtractorError(join_nonempty(error_code, error_info.get('message'), delim=': '))
            raise

        m3u8_url = traverse_obj(details, (
            ('adaptive_url', ('adaptive_urls', 'hd', 'hls', ..., 'playback_url')), {url_or_none}, any))
        formats, subtitles = self._extract_m3u8_formats_and_subtitles(m3u8_url, episode_slug, 'mp4')

        return {
            'formats': formats,
            'subtitles': subtitles,
            **self._extract_episode_info(metadata, episode_slug, series_slug),
        }


class DangalPlaySeasonIE(DangalPlayBaseIE):
    IE_NAME = 'dangalplay:season'
    _VALID_URL = r'https?://(?:www\.)?dangalplay.com/shows/(?P<id>[^/?#]+)(?:/(?P<sub>ep-[^/?#]+)/episodes)?/?(?:$|[?#])'
    _TESTS = [{
        'url': 'https://www.dangalplay.com/shows/kitani-mohabbat-hai-season-1',
        'playlist_mincount': 170,
        'info_dict': {
            'id': 'kitani-mohabbat-hai-season-1',
        },
    }, {
        'url': 'https://www.dangalplay.com/shows/kitani-mohabbat-hai-season-1/ep-01-30-1/episodes',
        'playlist_count': 30,
        'info_dict': {
            'id': 'kitani-mohabbat-hai-season-1-ep-01-30-1',
        },
    }, {
        # 1 season only, series page is season page
        'url': 'https://www.dangalplay.com/shows/milke-bhi-hum-na-mile',
        'playlist_mincount': 15,
        'info_dict': {
            'id': 'milke-bhi-hum-na-mile',
        },
    }]

    def _entries(self, subcategories, series_slug):
        for subcategory in subcategories:
            data = self._call_api(
                f'catalogs/shows/items/{series_slug}/subcategories/{subcategory}/episodes.gzip',
                series_slug, f'Downloading episodes JSON for {subcategory}', fatal=False, query={
                    'order_by': 'asc',
                    'status': 'published',
                })
            for ep in traverse_obj(data, ('data', 'items', lambda _, v: v['friendly_id'])):
                episode_slug = ep['friendly_id']
                yield self.url_result(
                    f'https://www.dangalplay.com/shows/{series_slug}/{episode_slug}',
                    DangalPlayIE, **self._extract_episode_info(ep, episode_slug, series_slug))

    def _real_extract(self, url):
        series_slug, subcategory = self._match_valid_url(url).group('id', 'sub')
        subcategories = [subcategory] if subcategory else traverse_obj(
            self._call_api(
                f'catalogs/shows/items/{series_slug}.gzip', series_slug,
                'Downloading season info JSON', query={'item_language': ''}),
            ('data', 'subcategories', ..., 'friendly_id', {str}))

        return self.playlist_result(
            self._entries(subcategories, series_slug), join_nonempty(series_slug, subcategory))



from ._tab import YoutubeTabBaseInfoExtractor
from ._video import YoutubeIE
from ...utils import ExtractorError, traverse_obj


class YoutubeClipIE(YoutubeTabBaseInfoExtractor):
    IE_NAME = 'youtube:clip'
    _VALID_URL = r'https?://(?:www\.)?youtube\.com/clip/(?P<id>[^/?#]+)'
    _TESTS = [{
        # FIXME: Other metadata should be extracted from the clip, not from the base video
        'url': 'https://www.youtube.com/clip/UgytZKpehg-hEMBSn3F4AaABCQ',
        'info_dict': {
            'id': 'UgytZKpehg-hEMBSn3F4AaABCQ',
            'ext': 'mp4',
            'section_start': 29.0,
            'section_end': 39.7,
            'duration': 10.7,
            'age_limit': 0,
            'availability': 'public',
            'categories': ['Gaming'],
            'channel': 'Scott The Woz',
            'channel_id': 'UC4rqhyiTs7XyuODcECvuiiQ',
            'channel_url': 'https://www.youtube.com/channel/UC4rqhyiTs7XyuODcECvuiiQ',
            'description': 'md5:7a4517a17ea9b4bd98996399d8bb36e7',
            'like_count': int,
            'playable_in_embed': True,
            'tags': 'count:17',
            'thumbnail': 'https://i.ytimg.com/vi_webp/ScPX26pdQik/maxresdefault.webp',
            'title': 'Mobile Games on Console - Scott The Woz',
            'upload_date': '20210920',
            'uploader': 'Scott The Woz',
            'uploader_id': '@ScottTheWoz',
            'uploader_url': 'https://www.youtube.com/@ScottTheWoz',
            'view_count': int,
            'live_status': 'not_live',
            'channel_follower_count': int,
            'chapters': 'count:20',
            'comment_count': int,
            'heatmap': 'count:100',
            'media_type': 'clip',
        },
    }]

    def _real_extract(self, url):
        clip_id = self._match_id(url)
        _, data = self._extract_webpage(url, clip_id)

        video_id = traverse_obj(data, ('currentVideoEndpoint', 'watchEndpoint', 'videoId'))
        if not video_id:
            raise ExtractorError('Unable to find video ID')

        clip_data = traverse_obj(data, (
            'engagementPanels', ..., 'engagementPanelSectionListRenderer', 'content', 'clipSectionRenderer',
            'contents', ..., 'clipAttributionRenderer', 'onScrubExit', 'commandExecutorCommand', 'commands', ...,
            'openPopupAction', 'popup', 'notificationActionRenderer', 'actionButton', 'buttonRenderer', 'command',
            'commandExecutorCommand', 'commands', ..., 'loopCommand'), get_all=False)

        return {
            '_type': 'url_transparent',
            'url': f'https://www.youtube.com/watch?v={video_id}',
            'ie_key': YoutubeIE.ie_key(),
            'id': clip_id,
            'media_type': 'clip',
            'section_start': int(clip_data['startTimeMs']) / 1000,
            'section_end': int(clip_data['endTimeMs']) / 1000,
            '_format_sort_fields': (  # https protocol is prioritized for ffmpeg compatibility
                'proto:https', 'quality', 'res', 'fps', 'hdr:12', 'source', 'vcodec', 'channels', 'acodec', 'lang'),
        }



from .common import InfoExtractor
from ..utils import (
    ExtractorError,
    mimetype2ext,
    url_or_none,
)
from ..utils.traversal import traverse_obj


class VidlyIE(InfoExtractor):
    _VALID_URL = r'https?://(?:vid\.ly/|(?:s\.)?vid\.ly/embeded\.html\?(?:[^#]+&)?link=)(?P<id>\w+)'
    _EMBED_REGEX = [r'<script[^>]+\bsrc=[\'"](?P<url>(?:https?:)?//vid\.ly/\w+/embed[^\'"]+)',
                    r'<iframe[^>]+\bsrc=[\'"](?P<url>(?:https?:)?//(?:s\.)?vid\.ly/embeded\.html\?(?:[^#\'"]+&)?link=\w+[^\'"]+)']
    _TESTS = [{
        # JWPlayer 7, Embeds forbidden
        'url': 'https://vid.ly/2i3o9j/embed',
        'info_dict': {
            'id': '2i3o9j',
            'ext': 'mp4',
            'title': '2i3o9j',
            'thumbnail': r're:https://\w+\.cloudfront\.net/',
        },
    }, {
        # JWPlayer 6
        'url': 'http://s.vid.ly/embeded.html?link=jw_test&new=1&autoplay=true&controls=true',
        'info_dict': {
            'id': 'jw_test',
            'ext': 'mp4',
            'title': '2x8m8t',
            'thumbnail': r're:https://\w+\.cloudfront\.net/',
        },
    }, {
        # Vidlyplayer
        'url': 'https://vid.ly/7x0e6l',
        'info_dict': {
            'id': '7x0e6l',
            'ext': 'mp4',
            'title': '7x0e6l',
        },
    }]
    _WEBPAGE_TESTS = [{
        'url': 'https://www.petfinder.com/dog/gus-57378930/tn/ooltewah/furever-furkids-rescue-tn592/',
        'info_dict': {
            'id': 'w8p5b0',
            'ext': 'mp4',
            'title': 'w8p5b0',
            'thumbnail': r're:https://\w+\.cloudfront\.net/',
        },
    }]

    def _real_extract(self, url):
        video_id = self._match_id(url)

        embed_script = self._download_webpage(
            f'https://vid.ly/{video_id}/embed', video_id, headers={'Referer': 'https://vid.ly/'})
        player = self._search_json(r'initCallback\(', embed_script, 'player', video_id)

        player_type = player.get('player') or ''
        if player_type.startswith('jwplayer'):
            return self._parse_jwplayer_data(player['config'], video_id)
        elif not player_type.startswith('vidly'):
            raise ExtractorError(f'Unknown player type {player_type!r}')

        formats = []
        ext = mimetype2ext(traverse_obj(player, ('config', 'type')))
        for source, fid in [('source', 'sd'), ('source_hd', 'hd')]:
            if traverse_obj(player, ('config', source, {url_or_none})):
                formats.append({
                    'url': player['config'][source],
                    'format_id': f'http-{fid}',
                    'ext': ext,
                })
        # Has higher quality formats
        formats.extend(self._extract_m3u8_formats(
            f'https://d3fenhwk93s16g.cloudfront.net/{video_id}/hls.m3u8', video_id,
            fatal=False, note='Requesting higher quality m3u8 formats',
            errnote='No higher quality m3u8 formats found') or [])

        return {
            'id': video_id,
            'title': video_id,
            'formats': formats,
        }



import functools
import re

from .common import InfoExtractor
from ..utils import (
    OnDemandPagedList,
    format_field,
    int_or_none,
    parse_resolution,
    str_or_none,
    try_get,
    unified_timestamp,
    url_or_none,
    urljoin,
)


class PeerTubeIE(InfoExtractor):
    _INSTANCES_RE = r'''(?:
                            # Taken from https://instances.joinpeertube.org/instances
                            0ch\.tv|
                            3dctube\.3dcandy\.social|
                            all\.electric\.kitchen|
                            alterscope\.fr|
                            anarchy\.tube|
                            apathy\.tv|
                            apertatube\.net|
                            archive\.nocopyrightintended\.tv|
                            archive\.reclaim\.tv|
                            area51\.media|
                            astrotube-ufe\.obspm\.fr|
                            astrotube\.obspm\.fr|
                            audio\.freediverse\.com|
                            azxtube\.youssefc\.tn|
                            bark\.video|
                            battlepenguin\.video|
                            bava\.tv|
                            bee-tube\.fr|
                            beetoons\.tv|
                            biblion\.refchat\.net|
                            biblioteca\.theowlclub\.net|
                            bideoak\.argia\.eus|
                            bideoteka\.eus|
                            birdtu\.be|
                            bitcointv\.com|
                            bonn\.video|
                            breeze\.tube|
                            brioco\.live|
                            brocosoup\.fr|
                            canal\.facil\.services|
                            canard\.tube|
                            cdn01\.tilvids\.com|
                            celluloid-media\.huma-num\.fr|
                            chicago1\.peertube\.support|
                            cliptube\.org|
                            cloudtube\.ise\.fraunhofer\.de|
                            comf\.tube|
                            comics\.peertube\.biz|
                            commons\.tube|
                            communitymedia\.video|
                            conspiracydistillery\.com|
                            crank\.recoil\.org|
                            dalek\.zone|
                            dalliance\.network|
                            dangly\.parts|
                            darkvapor\.nohost\.me|
                            daschauher\.aksel\.rocks|
                            digitalcourage\.video|
                            displayeurope\.video|
                            ds106\.tv|
                            dud-video\.inf\.tu-dresden\.de|
                            dud175\.inf\.tu-dresden\.de|
                            dytube\.com|
                            ebildungslabor\.video|
                            evangelisch\.video|
                            fair\.tube|
                            fedi\.video|
                            fedimovie\.com|
                            fediverse\.tv|
                            film\.k-prod\.fr|
                            flipboard\.video|
                            foss\.video|
                            fossfarmers\.company|
                            fotogramas\.politicaconciencia\.org|
                            freediverse\.com|
                            freesoto-u2151\.vm\.elestio\.app|
                            freesoto\.tv|
                            garr\.tv|
                            greatview\.video|
                            grypstube\.uni-greifswald\.de|
                            habratube\.site|
                            ilbjach\.ru|
                            infothema\.net|
                            itvplus\.iiens\.net|
                            johnydeep\.net|
                            juggling\.digital|
                            jupiter\.tube|
                            kadras\.live|
                            kino\.kompot\.si|
                            kino\.schuerz\.at|
                            kinowolnosc\.pl|
                            kirche\.peertube-host\.de|
                            kiwi\.froggirl\.club|
                            kodcast\.com|
                            kolektiva\.media|
                            kpop\.22x22\.ru|
                            kumi\.tube|
                            la2\.peertube\.support|
                            la3\.peertube\.support|
                            la4\.peertube\.support|
                            lastbreach\.tv|
                            lawsplaining\.peertube\.biz|
                            leopard\.tube|
                            live\.codinglab\.ch|
                            live\.libratoi\.org|
                            live\.oldskool\.fi|
                            live\.solari\.com|
                            lucarne\.balsamine\.be|
                            luxtube\.lu|
                            makertube\.net|
                            media\.econoalchemist\.com|
                            media\.exo\.cat|
                            media\.fsfe\.org|
                            media\.gzevd\.de|
                            media\.interior\.edu\.uy|
                            media\.krashboyz\.org|
                            media\.mzhd\.de|
                            media\.smz-ma\.de|
                            media\.theplattform\.net|
                            media\.undeadnetwork\.de|
                            medias\.debrouillonet\.org|
                            medias\.pingbase\.net|
                            mediatube\.fermalo\.fr|
                            melsungen\.peertube-host\.de|
                            merci-la-police\.fr|
                            mindlyvideos\.com|
                            mirror\.peertube\.metalbanana\.net|
                            mirrored\.rocks|
                            mix\.video|
                            mountaintown\.video|
                            movies\.metricsmaster\.eu|
                            mtube\.mooo\.com|
                            mytube\.kn-cloud\.de|
                            mytube\.le5emeaxe\.fr|
                            mytube\.madzel\.de|
                            nadajemy\.com|
                            nanawel-peertube\.dyndns\.org|
                            neat\.tube|
                            nethack\.tv|
                            nicecrew\.tv|
                            nightshift\.minnix\.dev|
                            nolog\.media|
                            nyltube\.nylarea\.com|
                            ocfedtest\.hosted\.spacebear\.ee|
                            openmedia\.edunova\.it|
                            p2ptv\.ru|
                            p\.eertu\.be|
                            p\.lu|
                            pastafriday\.club|
                            patriottube\.sonsofliberty\.red|
                            pcbu\.nl|
                            peer\.azurs\.fr|
                            peer\.d0g4\.me|
                            peer\.lukeog\.com|
                            peer\.madiator\.cloud|
                            peer\.raise-uav\.com|
                            peershare\.togart\.de|
                            peertube-blablalinux\.be|
                            peertube-demo\.learning-hub\.fr|
                            peertube-docker\.cpy\.re|
                            peertube-eu\.howlround\.com|
                            peertube-u5014\.vm\.elestio\.app|
                            peertube-us\.howlround\.com|
                            peertube\.020\.pl|
                            peertube\.0x5e\.eu|
                            peertube\.1984\.cz|
                            peertube\.2i2l\.net|
                            peertube\.adjutor\.xyz|
                            peertube\.adresse\.data\.gouv\.fr|
                            peertube\.alpharius\.io|
                            peertube\.am-networks\.fr|
                            peertube\.anduin\.net|
                            peertube\.anti-logic\.com|
                            peertube\.arch-linux\.cz|
                            peertube\.art3mis\.de|
                            peertube\.artsrn\.ualberta\.ca|
                            peertube\.askan\.info|
                            peertube\.astral0pitek\.synology\.me|
                            peertube\.atsuchan\.page|
                            peertube\.automat\.click|
                            peertube\.b38\.rural-it\.org|
                            peertube\.be|
                            peertube\.beeldengeluid\.nl|
                            peertube\.bgzashtita\.es|
                            peertube\.bike|
                            peertube\.bildung-ekhn\.de|
                            peertube\.biz|
                            peertube\.br0\.fr|
                            peertube\.bridaahost\.ynh\.fr|
                            peertube\.bubbletea\.dev|
                            peertube\.bubuit\.net|
                            peertube\.cabaal\.net|
                            peertube\.chatinbit\.com|
                            peertube\.chaunchy\.com|
                            peertube\.chir\.rs|
                            peertube\.christianpacaud\.com|
                            peertube\.chtisurel\.net|
                            peertube\.chuggybumba\.com|
                            peertube\.cipherbliss\.com|
                            peertube\.cirkau\.art|
                            peertube\.cloud\.nerdraum\.de|
                            peertube\.cloud\.sans\.pub|
                            peertube\.coko\.foundation|
                            peertube\.communecter\.org|
                            peertube\.concordia\.social|
                            peertube\.corrigan\.xyz|
                            peertube\.cpge-brizeux\.fr|
                            peertube\.ctseuro\.com|
                            peertube\.cuatrolibertades\.org|
                            peertube\.cube4fun\.net|
                            peertube\.dair-institute\.org|
                            peertube\.davigge\.com|
                            peertube\.dc\.pini\.fr|
                            peertube\.deadtom\.me|
                            peertube\.debian\.social|
                            peertube\.delta0189\.xyz|
                            peertube\.demonix\.fr|
                            peertube\.designersethiques\.org|
                            peertube\.desmu\.fr|
                            peertube\.devol\.it|
                            peertube\.dk|
                            peertube\.doesstuff\.social|
                            peertube\.eb8\.org|
                            peertube\.education-forum\.com|
                            peertube\.elforcer\.ru|
                            peertube\.em\.id\.lv|
                            peertube\.ethibox\.fr|
                            peertube\.eu\.org|
                            peertube\.european-pirates\.eu|
                            peertube\.eus|
                            peertube\.euskarabildua\.eus|
                            peertube\.expi\.studio|
                            peertube\.familie-berner\.de|
                            peertube\.familleboisteau\.fr|
                            peertube\.fedihost\.website|
                            peertube\.fenarinarsa\.com|
                            peertube\.festnoz\.de|
                            peertube\.forteza\.fr|
                            peertube\.freestorm\.online|
                            peertube\.functional\.cafe|
                            peertube\.gaminglinux\.fr|
                            peertube\.gargantia\.fr|
                            peertube\.geekgalaxy\.fr|
                            peertube\.gemlog\.ca|
                            peertube\.genma\.fr|
                            peertube\.get-racing\.de|
                            peertube\.ghis94\.ovh|
                            peertube\.gidikroon\.eu|
                            peertube\.giftedmc\.com|
                            peertube\.grosist\.fr|
                            peertube\.gruntwerk\.org|
                            peertube\.gsugambit\.com|
                            peertube\.hackerfoo\.com|
                            peertube\.hellsite\.net|
                            peertube\.helvetet\.eu|
                            peertube\.histoirescrepues\.fr|
                            peertube\.home\.x0r\.fr|
                            peertube\.hyperfreedom\.org|
                            peertube\.ichigo\.everydayimshuflin\.com|
                            peertube\.ifwo\.eu|
                            peertube\.in\.ua|
                            peertube\.inapurna\.org|
                            peertube\.informaction\.info|
                            peertube\.interhop\.org|
                            peertube\.it|
                            peertube\.it-arts\.net|
                            peertube\.jensdiemer\.de|
                            peertube\.johntheserg\.al|
                            peertube\.kaleidos\.net|
                            peertube\.kalua\.im|
                            peertube\.kcore\.org|
                            peertube\.keazilla\.net|
                            peertube\.klaewyss\.fr|
                            peertube\.kleph\.eu|
                            peertube\.kodein\.be|
                            peertube\.kooperatywa\.tech|
                            peertube\.kriom\.net|
                            peertube\.kx\.studio|
                            peertube\.kyriog\.eu|
                            peertube\.la-famille-muller\.fr|
                            peertube\.labeuropereunion\.eu|
                            peertube\.lagvoid\.com|
                            peertube\.lhc\.net\.br|
                            peertube\.libresolutions\.network|
                            peertube\.libretic\.fr|
                            peertube\.librosphere\.fr|
                            peertube\.logilab\.fr|
                            peertube\.lon\.tv|
                            peertube\.louisematic\.site|
                            peertube\.luckow\.org|
                            peertube\.luga\.at|
                            peertube\.lyceeconnecte\.fr|
                            peertube\.madixam\.xyz|
                            peertube\.magicstone\.dev|
                            peertube\.marienschule\.de|
                            peertube\.marud\.fr|
                            peertube\.maxweiss\.io|
                            peertube\.miguelcr\.me|
                            peertube\.mikemestnik\.net|
                            peertube\.mobilsicher\.de|
                            peertube\.monlycee\.net|
                            peertube\.mxinfo\.fr|
                            peertube\.naln1\.ca|
                            peertube\.netzbegruenung\.de|
                            peertube\.nicolastissot\.fr|
                            peertube\.nogafam\.fr|
                            peertube\.normalgamingcommunity\.cz|
                            peertube\.nz|
                            peertube\.offerman\.com|
                            peertube\.ohioskates\.com|
                            peertube\.onionstorm\.net|
                            peertube\.opencloud\.lu|
                            peertube\.otakufarms\.com|
                            peertube\.paladyn\.org|
                            peertube\.pix-n-chill\.fr|
                            peertube\.r2\.enst\.fr|
                            peertube\.r5c3\.fr|
                            peertube\.redpill-insight\.com|
                            peertube\.researchinstitute\.at|
                            peertube\.revelin\.fr|
                            peertube\.rlp\.schule|
                            peertube\.rokugan\.fr|
                            peertube\.rougevertbleu\.tv|
                            peertube\.roundpond\.net|
                            peertube\.rural-it\.org|
                            peertube\.satoshishop\.de|
                            peertube\.scyldings\.com|
                            peertube\.securitymadein\.lu|
                            peertube\.semperpax\.com|
                            peertube\.semweb\.pro|
                            peertube\.sensin\.eu|
                            peertube\.sidh\.bzh|
                            peertube\.skorpil\.cz|
                            peertube\.smertrios\.com|
                            peertube\.sqweeb\.net|
                            peertube\.stattzeitung\.org|
                            peertube\.stream|
                            peertube\.su|
                            peertube\.swrs\.net|
                            peertube\.takeko\.cyou|
                            peertube\.taxinachtegel\.de|
                            peertube\.teftera\.com|
                            peertube\.teutronic-services\.de|
                            peertube\.ti-fr\.com|
                            peertube\.tiennot\.net|
                            peertube\.tmp\.rcp\.tf|
                            peertube\.tspu\.edu\.ru|
                            peertube\.tv|
                            peertube\.tweb\.tv|
                            peertube\.underworld\.fr|
                            peertube\.vapronva\.pw|
                            peertube\.veen\.world|
                            peertube\.vesdia\.eu|
                            peertube\.virtual-assembly\.org|
                            peertube\.viviers-fibre\.net|
                            peertube\.vlaki\.cz|
                            peertube\.wiesbaden\.social|
                            peertube\.wivodaim\.net|
                            peertube\.wtf|
                            peertube\.wtfayla\.net|
                            peertube\.xrcb\.cat|
                            peertube\.xwiki\.com|
                            peertube\.zd\.do|
                            peertube\.zetamc\.net|
                            peertube\.zmuuf\.org|
                            peertube\.zoz-serv\.org|
                            peertube\.zwindler\.fr|
                            peervideo\.ru|
                            periscope\.numenaute\.org|
                            pete\.warpnine\.de|
                            petitlutinartube\.fr|
                            phijkchu\.com|
                            phoenixproject\.group|
                            piraten\.space|
                            pirtube\.calut\.fr|
                            pityu\.flaki\.hu|
                            play\.mittdata\.se|
                            player\.ojamajo\.moe|
                            podlibre\.video|
                            portal\.digilab\.nfa\.cz|
                            private\.fedimovie\.com|
                            pt01\.lehrerfortbildung-bw\.de|
                            pt\.diaspodon\.fr|
                            pt\.freedomwolf\.cc|
                            pt\.gordons\.gen\.nz|
                            pt\.ilyamikcoder\.com|
                            pt\.irnok\.net|
                            pt\.mezzo\.moe|
                            pt\.na4\.eu|
                            pt\.netcraft\.ch|
                            pt\.rwx\.ch|
                            pt\.sfunk1x\.com|
                            pt\.thishorsie\.rocks|
                            pt\.vern\.cc|
                            ptb\.lunarviews\.net|
                            ptube\.de|
                            ptube\.ranranhome\.info|
                            puffy\.tube|
                            puppet\.zone|
                            qtube\.qlyoung\.net|
                            quantube\.win|
                            rankett\.net|
                            replay\.jres\.org|
                            review\.peertube\.biz|
                            sdmtube\.fr|
                            secure\.direct-live\.net|
                            secure\.scanovid\.com|
                            seka\.pona\.la|
                            serv3\.wiki-tube\.de|
                            skeptube\.fr|
                            social\.fedimovie\.com|
                            socpeertube\.ru|
                            sovran\.video|
                            special\.videovortex\.tv|
                            spectra\.video|
                            stl1988\.peertube-host\.de|
                            stream\.biovisata\.lt|
                            stream\.conesphere\.cloud|
                            stream\.elven\.pw|
                            stream\.jurnalfm\.md|
                            stream\.k-prod\.fr|
                            stream\.litera\.tools|
                            stream\.nuemedia\.se|
                            stream\.rlp-media\.de|
                            stream\.vrse\.be|
                            studios\.racer159\.com|
                            styxhexenhammer666\.com|
                            syrteplay\.obspm\.fr|
                            t\.0x0\.st|
                            tbh\.co-shaoghal\.net|
                            test-fab\.ynh\.fr|
                            testube\.distrilab\.fr|
                            tgi\.hosted\.spacebear\.ee|
                            theater\.ethernia\.net|
                            thecool\.tube|
                            thevideoverse\.com|
                            tilvids\.com|
                            tinkerbetter\.tube|
                            tinsley\.video|
                            trailers\.ddigest\.com|
                            tube-action-educative\.apps\.education\.fr|
                            tube-arts-lettres-sciences-humaines\.apps\.education\.fr|
                            tube-cycle-2\.apps\.education\.fr|
                            tube-cycle-3\.apps\.education\.fr|
                            tube-education-physique-et-sportive\.apps\.education\.fr|
                            tube-enseignement-professionnel\.apps\.education\.fr|
                            tube-institutionnel\.apps\.education\.fr|
                            tube-langues-vivantes\.apps\.education\.fr|
                            tube-maternelle\.apps\.education\.fr|
                            tube-numerique-educatif\.apps\.education\.fr|
                            tube-sciences-technologies\.apps\.education\.fr|
                            tube-test\.apps\.education\.fr|
                            tube1\.perron-service\.de|
                            tube\.9minuti\.it|
                            tube\.abolivier\.bzh|
                            tube\.alado\.space|
                            tube\.amic37\.fr|
                            tube\.area404\.cloud|
                            tube\.arthack\.nz|
                            tube\.asulia\.fr|
                            tube\.awkward\.company|
                            tube\.azbyka\.ru|
                            tube\.azkware\.net|
                            tube\.bartrip\.me\.uk|
                            tube\.belowtoxic\.media|
                            tube\.bingle\.plus|
                            tube\.bit-friends\.de|
                            tube\.bstly\.de|
                            tube\.chosto\.me|
                            tube\.cms\.garden|
                            tube\.communia\.org|
                            tube\.cyberia\.club|
                            tube\.cybershock\.life|
                            tube\.dembased\.xyz|
                            tube\.dev\.displ\.eu|
                            tube\.digitalesozialearbeit\.de|
                            tube\.distrilab\.fr|
                            tube\.doortofreedom\.org|
                            tube\.dsocialize\.net|
                            tube\.e-jeremy\.com|
                            tube\.ebin\.club|
                            tube\.elemac\.fr|
                            tube\.erzbistum-hamburg\.de|
                            tube\.exozy\.me|
                            tube\.fdn\.fr|
                            tube\.fedi\.quebec|
                            tube\.fediverse\.at|
                            tube\.felinn\.org|
                            tube\.flokinet\.is|
                            tube\.foad\.me\.uk|
                            tube\.freepeople\.fr|
                            tube\.friloux\.me|
                            tube\.froth\.zone|
                            tube\.fulda\.social|
                            tube\.futuretic\.fr|
                            tube\.g1zm0\.de|
                            tube\.g4rf\.net|
                            tube\.gaiac\.io|
                            tube\.geekyboo\.net|
                            tube\.genb\.de|
                            tube\.ghk-academy\.info|
                            tube\.gi-it\.de|
                            tube\.grap\.coop|
                            tube\.graz\.social|
                            tube\.grin\.hu|
                            tube\.hokai\.lol|
                            tube\.int5\.net|
                            tube\.interhacker\.space|
                            tube\.invisible\.ch|
                            tube\.io18\.top|
                            tube\.itsg\.host|
                            tube\.jeena\.net|
                            tube\.kh-berlin\.de|
                            tube\.kockatoo\.org|
                            tube\.kotur\.org|
                            tube\.koweb\.fr|
                            tube\.la-dina\.net|
                            tube\.lab\.nrw|
                            tube\.lacaveatonton\.ovh|
                            tube\.laurent-malys\.fr|
                            tube\.leetdreams\.ch|
                            tube\.linkse\.media|
                            tube\.lokad\.com|
                            tube\.lucie-philou\.com|
                            tube\.media-techport\.de|
                            tube\.morozoff\.pro|
                            tube\.neshweb\.net|
                            tube\.nestor\.coop|
                            tube\.network\.europa\.eu|
                            tube\.nicfab\.eu|
                            tube\.nieuwwestbrabant\.nl|
                            tube\.nogafa\.org|
                            tube\.novg\.net|
                            tube\.nox-rhea\.org|
                            tube\.nuagelibre\.fr|
                            tube\.numerique\.gouv\.fr|
                            tube\.nuxnik\.com|
                            tube\.nx12\.net|
                            tube\.octaplex\.net|
                            tube\.oisux\.org|
                            tube\.okcinfo\.news|
                            tube\.onlinekirche\.net|
                            tube\.opportunis\.me|
                            tube\.oraclefilms\.com|
                            tube\.org\.il|
                            tube\.pacapime\.ovh|
                            tube\.parinux\.org|
                            tube\.pastwind\.top|
                            tube\.picasoft\.net|
                            tube\.pilgerweg-21\.de|
                            tube\.pmj\.rocks|
                            tube\.pol\.social|
                            tube\.ponsonaille\.fr|
                            tube\.portes-imaginaire\.org|
                            tube\.public\.apolut\.net|
                            tube\.pustule\.org|
                            tube\.pyngu\.com|
                            tube\.querdenken-711\.de|
                            tube\.rebellion\.global|
                            tube\.reseau-canope\.fr|
                            tube\.rhythms-of-resistance\.org|
                            tube\.risedsky\.ovh|
                            tube\.rooty\.fr|
                            tube\.rsi\.cnr\.it|
                            tube\.ryne\.moe|
                            tube\.schleuss\.online|
                            tube\.schule\.social|
                            tube\.sekretaerbaer\.net|
                            tube\.shanti\.cafe|
                            tube\.shela\.nu|
                            tube\.skrep\.in|
                            tube\.sleeping\.town|
                            tube\.sp-codes\.de|
                            tube\.spdns\.org|
                            tube\.systerserver\.net|
                            tube\.systest\.eu|
                            tube\.tappret\.fr|
                            tube\.techeasy\.org|
                            tube\.thierrytalbert\.fr|
                            tube\.tinfoil-hat\.net|
                            tube\.toldi\.eu|
                            tube\.tpshd\.de|
                            tube\.trax\.im|
                            tube\.troopers\.agency|
                            tube\.ttk\.is|
                            tube\.tuxfriend\.fr|
                            tube\.tylerdavis\.xyz|
                            tube\.ullihome\.de|
                            tube\.ulne\.be|
                            tube\.undernet\.uy|
                            tube\.vrpnet\.org|
                            tube\.wolfe\.casa|
                            tube\.xd0\.de|
                            tube\.xn--baw-joa\.social|
                            tube\.xy-space\.de|
                            tube\.yapbreak\.fr|
                            tubedu\.org|
                            tubulus\.openlatin\.org|
                            turtleisland\.video|
                            tututu\.tube|
                            tv\.adast\.dk|
                            tv\.adn\.life|
                            tv\.arns\.lt|
                            tv\.atmx\.ca|
                            tv\.based\.quest|
                            tv\.farewellutopia\.com|
                            tv\.filmfreedom\.net|
                            tv\.gravitons\.org|
                            tv\.io\.seg\.br|
                            tv\.lumbung\.space|
                            tv\.pirateradio\.social|
                            tv\.pirati\.cz|
                            tv\.santic-zombie\.ru|
                            tv\.undersco\.re|
                            tv\.zonepl\.net|
                            tvox\.ru|
                            twctube\.twc-zone\.eu|
                            twobeek\.com|
                            urbanists\.video|
                            v\.9tail\.net|
                            v\.basspistol\.org|
                            v\.j4\.lc|
                            v\.kisombrella\.top|
                            v\.koa\.im|
                            v\.kyaru\.xyz|
                            v\.lor\.sh|
                            v\.mkp\.ca|
                            v\.posm\.gay|
                            v\.slaycer\.top|
                            veedeo\.org|
                            vhs\.absturztau\.be|
                            vid\.cthos\.dev|
                            vid\.kinuseka\.us|
                            vid\.mkp\.ca|
                            vid\.nocogabriel\.fr|
                            vid\.norbipeti\.eu|
                            vid\.northbound\.online|
                            vid\.ohboii\.de|
                            vid\.plantplotting\.co\.uk|
                            vid\.pretok\.tv|
                            vid\.prometheus\.systems|
                            vid\.soafen\.love|
                            vid\.twhtv\.club|
                            vid\.wildeboer\.net|
                            video-cave-v2\.de|
                            video-liberty\.com|
                            video\.076\.ne\.jp|
                            video\.1146\.nohost\.me|
                            video\.9wd\.eu|
                            video\.abraum\.de|
                            video\.ados\.accoord\.fr|
                            video\.amiga-ng\.org|
                            video\.anartist\.org|
                            video\.asgardius\.company|
                            video\.audiovisuel-participatif\.org|
                            video\.bards\.online|
                            video\.barkoczy\.social|
                            video\.benetou\.fr|
                            video\.beyondwatts\.social|
                            video\.bgeneric\.net|
                            video\.bilecik\.edu\.tr|
                            video\.blast-info\.fr|
                            video\.bmu\.cloud|
                            video\.catgirl\.biz|
                            video\.causa-arcana\.com|
                            video\.chasmcity\.net|
                            video\.chbmeyer\.de|
                            video\.cigliola\.com|
                            video\.citizen4\.eu|
                            video\.clumsy\.computer|
                            video\.cnnumerique\.fr|
                            video\.cnr\.it|
                            video\.cnt\.social|
                            video\.coales\.co|
                            video\.comune\.trento\.it|
                            video\.coyp\.us|
                            video\.csc49\.fr|
                            video\.davduf\.net|
                            video\.davejansen\.com|
                            video\.dlearning\.nl|
                            video\.dnfi\.no|
                            video\.dresden\.network|
                            video\.drgnz\.club|
                            video\.dudenas\.lt|
                            video\.eientei\.org|
                            video\.ellijaymakerspace\.org|
                            video\.emergeheart\.info|
                            video\.eradicatinglove\.xyz|
                            video\.everythingbagel\.me|
                            video\.extremelycorporate\.ca|
                            video\.fabiomanganiello\.com|
                            video\.fedi\.bzh|
                            video\.fhtagn\.org|
                            video\.firehawk-systems\.com|
                            video\.fox-romka\.ru|
                            video\.fuss\.bz\.it|
                            video\.glassbeadcollective\.org|
                            video\.graine-pdl\.org|
                            video\.gyt\.is|
                            video\.hainry\.fr|
                            video\.hardlimit\.com|
                            video\.hostux\.net|
                            video\.igem\.org|
                            video\.infojournal\.fr|
                            video\.internet-czas-dzialac\.pl|
                            video\.interru\.io|
                            video\.ipng\.ch|
                            video\.ironsysadmin\.com|
                            video\.islameye\.com|
                            video\.jacen\.moe|
                            video\.jadin\.me|
                            video\.jeffmcbride\.net|
                            video\.jigmedatse\.com|
                            video\.kuba-orlik\.name|
                            video\.lacalligramme\.fr|
                            video\.lanceurs-alerte\.fr|
                            video\.laotra\.red|
                            video\.lapineige\.fr|
                            video\.laraffinerie\.re|
                            video\.lavolte\.net|
                            video\.liberta\.vip|
                            video\.libreti\.net|
                            video\.licentia\.net|
                            video\.linc\.systems|
                            video\.linux\.it|
                            video\.linuxtrent\.it|
                            video\.liveitlive\.show|
                            video\.lono\.space|
                            video\.lrose\.de|
                            video\.lunago\.net|
                            video\.lundi\.am|
                            video\.lycee-experimental\.org|
                            video\.maechler\.cloud|
                            video\.marcorennmaus\.de|
                            video\.mass-trespass\.uk|
                            video\.matomocamp\.org|
                            video\.medienzentrum-harburg\.de|
                            video\.mentality\.rip|
                            video\.metaversum\.wtf|
                            video\.midreality\.com|
                            video\.mttv\.it|
                            video\.mugoreve\.fr|
                            video\.mxtthxw\.art|
                            video\.mycrowd\.ca|
                            video\.niboe\.info|
                            video\.nogafam\.es|
                            video\.nstr\.no|
                            video\.occm\.cc|
                            video\.off-investigation\.fr|
                            video\.olos311\.org|
                            video\.ordinobsolete\.fr|
                            video\.osvoj\.ru|
                            video\.ourcommon\.cloud|
                            video\.ozgurkon\.org|
                            video\.pcf\.fr|
                            video\.pcgaldo\.com|
                            video\.phyrone\.de|
                            video\.poul\.org|
                            video\.publicspaces\.net|
                            video\.pullopen\.xyz|
                            video\.r3s\.nrw|
                            video\.rainevixen\.com|
                            video\.resolutions\.it|
                            video\.retroedge\.tech|
                            video\.rhizome\.org|
                            video\.rlp-media\.de|
                            video\.rs-einrich\.de|
                            video\.rubdos\.be|
                            video\.sadmin\.io|
                            video\.sftblw\.moe|
                            video\.shitposter\.club|
                            video\.simplex-software\.ru|
                            video\.slipfox\.xyz|
                            video\.snug\.moe|
                            video\.software-fuer-engagierte\.de|
                            video\.soi\.ch|
                            video\.sonet\.ws|
                            video\.surazal\.net|
                            video\.taskcards\.eu|
                            video\.team-lcbs\.eu|
                            video\.techforgood\.social|
                            video\.telemillevaches\.net|
                            video\.thepolarbear\.co\.uk|
                            video\.thinkof\.name|
                            video\.tii\.space|
                            video\.tkz\.es|
                            video\.trankil\.info|
                            video\.triplea\.fr|
                            video\.tum\.social|
                            video\.turbo\.chat|
                            video\.uriopss-pdl\.fr|
                            video\.ustim\.ru|
                            video\.ut0pia\.org|
                            video\.vaku\.org\.ua|
                            video\.vegafjord\.me|
                            video\.veloma\.org|
                            video\.violoncello\.ch|
                            video\.voidconspiracy\.band|
                            video\.wakkeren\.nl|
                            video\.windfluechter\.org|
                            video\.ziez\.eu|
                            videos-passages\.huma-num\.fr|
                            videos\.aadtp\.be|
                            videos\.ahp-numerique\.fr|
                            videos\.alamaisondulibre\.org|
                            videos\.archigny\.net|
                            videos\.aroaduntraveled\.com|
                            videos\.b4tech\.org|
                            videos\.benjaminbrady\.ie|
                            videos\.bik\.opencloud\.lu|
                            videos\.cloudron\.io|
                            videos\.codingotaku\.com|
                            videos\.coletivos\.org|
                            videos\.collate\.social|
                            videos\.danksquad\.org|
                            videos\.digitaldragons\.eu|
                            videos\.dromeadhere\.fr|
                            videos\.explain-it\.org|
                            videos\.factsonthegroundshow\.com|
                            videos\.foilen\.com|
                            videos\.fsci\.in|
                            videos\.gamercast\.net|
                            videos\.gianmarco\.gg|
                            videos\.globenet\.org|
                            videos\.grafo\.zone|
                            videos\.hauspie\.fr|
                            videos\.hush\.is|
                            videos\.hyphalfusion\.network|
                            videos\.icum\.to|
                            videos\.im\.allmendenetz\.de|
                            videos\.jacksonchen666\.com|
                            videos\.john-livingston\.fr|
                            videos\.knazarov\.com|
                            videos\.kuoushi\.com|
                            videos\.laliguepaysdelaloire\.org|
                            videos\.lemouvementassociatif-pdl\.org|
                            videos\.leslionsfloorball\.fr|
                            videos\.librescrum\.org|
                            videos\.mastodont\.cat|
                            videos\.metus\.ca|
                            videos\.miolo\.org|
                            videos\.offroad\.town|
                            videos\.openmandriva\.org|
                            videos\.parleur\.net|
                            videos\.pcorp\.us|
                            videos\.pop\.eu\.com|
                            videos\.rampin\.org|
                            videos\.rauten\.co\.za|
                            videos\.ritimo\.org|
                            videos\.sarcasmstardust\.com|
                            videos\.scanlines\.xyz|
                            videos\.shmalls\.pw|
                            videos\.stadtfabrikanten\.org|
                            videos\.supertuxkart\.net|
                            videos\.testimonia\.org|
                            videos\.thinkerview\.com|
                            videos\.torrenezzi10\.xyz|
                            videos\.trom\.tf|
                            videos\.utsukta\.org|
                            videos\.viorsan\.com|
                            videos\.wherelinux\.xyz|
                            videos\.wikilibriste\.fr|
                            videos\.yesil\.club|
                            videos\.yeswiki\.net|
                            videotube\.duckdns\.org|
                            vids\.capypara\.de|
                            vids\.roshless\.me|
                            vids\.stary\.pc\.pl|
                            vids\.tekdmn\.me|
                            vidz\.julien\.ovh|
                            views\.southfox\.me|
                            virtual-girls-are\.definitely-for\.me|
                            viste\.pt|
                            vnchich\.com|
                            vnop\.org|
                            vod\.newellijay\.tv|
                            voluntarytube\.com|
                            vtr\.chikichiki\.tube|
                            vulgarisation-informatique\.fr|
                            watch\.easya\.solutions|
                            watch\.goodluckgabe\.life|
                            watch\.ignorance\.eu|
                            watch\.jimmydore\.com|
                            watch\.libertaria\.space|
                            watch\.nuked\.social|
                            watch\.ocaml\.org|
                            watch\.thelema\.social|
                            watch\.tubelab\.video|
                            web-fellow\.de|
                            webtv\.vandoeuvre\.net|
                            wetubevid\.online|
                            wikileaks\.video|
                            wiwi\.video|
                            wow\.such\.disappointment\.fail|
                            www\.jvideos\.net|
                            www\.kotikoff\.net|
                            www\.makertube\.net|
                            www\.mypeer\.tube|
                            www\.nadajemy\.com|
                            www\.neptube\.io|
                            www\.rocaguinarda\.tv|
                            www\.vnshow\.net|
                            xxivproduction\.video|
                            yt\.orokoro\.ru|
                            ytube\.retronerd\.at|
                            zumvideo\.de|

                            # from youtube-dl
                            peertube\.rainbowswingers\.net|
                            tube\.stanisic\.nl|
                            peer\.suiri\.us|
                            medias\.libox\.fr|
                            videomensoif\.ynh\.fr|
                            peertube\.travelpandas\.eu|
                            peertube\.rachetjay\.fr|
                            peertube\.montecsys\.fr|
                            tube\.eskuero\.me|
                            peer\.tube|
                            peertube\.umeahackerspace\.se|
                            tube\.nx-pod\.de|
                            video\.monsieurbidouille\.fr|
                            tube\.openalgeria\.org|
                            vid\.lelux\.fi|
                            video\.anormallostpod\.ovh|
                            tube\.crapaud-fou\.org|
                            peertube\.stemy\.me|
                            lostpod\.space|
                            exode\.me|
                            peertube\.snargol\.com|
                            vis\.ion\.ovh|
                            videosdulib\.re|
                            v\.mbius\.io|
                            videos\.judrey\.eu|
                            peertube\.osureplayviewer\.xyz|
                            peertube\.mathieufamily\.ovh|
                            www\.videos-libr\.es|
                            fightforinfo\.com|
                            peertube\.fediverse\.ru|
                            peertube\.oiseauroch\.fr|
                            video\.nesven\.eu|
                            v\.bearvideo\.win|
                            video\.qoto\.org|
                            justporn\.cc|
                            video\.vny\.fr|
                            peervideo\.club|
                            tube\.taker\.fr|
                            peertube\.chantierlibre\.org|
                            tube\.ipfixe\.info|
                            tube\.kicou\.info|
                            tube\.dodsorf\.as|
                            videobit\.cc|
                            video\.yukari\.moe|
                            videos\.elbinario\.net|
                            hkvideo\.live|
                            pt\.tux\.tf|
                            www\.hkvideo\.live|
                            FIGHTFORINFO\.com|
                            pt\.765racing\.com|
                            peertube\.gnumeria\.eu\.org|
                            nordenmedia\.com|
                            peertube\.co\.uk|
                            tube\.darfweb\.eu|
                            tube\.kalah-france\.org|
                            0ch\.in|
                            vod\.mochi\.academy|
                            film\.node9\.org|
                            peertube\.hatthieves\.es|
                            video\.fitchfamily\.org|
                            peertube\.ddns\.net|
                            video\.ifuncle\.kr|
                            video\.fdlibre\.eu|
                            tube\.22decembre\.eu|
                            peertube\.harmoniescreatives\.com|
                            tube\.fabrigli\.fr|
                            video\.thedwyers\.co|
                            video\.bruitbruit\.com|
                            peertube\.foxfam\.club|
                            peer\.philoxweb\.be|
                            videos\.bugs\.social|
                            peertube\.malbert\.xyz|
                            peertube\.bilange\.ca|
                            libretube\.net|
                            diytelevision\.com|
                            peertube\.fedilab\.app|
                            libre\.video|
                            video\.mstddntfdn\.online|
                            us\.tv|
                            peertube\.sl-network\.fr|
                            peertube\.dynlinux\.io|
                            peertube\.david\.durieux\.family|
                            peertube\.linuxrocks\.online|
                            peerwatch\.xyz|
                            v\.kretschmann\.social|
                            tube\.otter\.sh|
                            yt\.is\.nota\.live|
                            tube\.dragonpsi\.xyz|
                            peertube\.boneheadmedia\.com|
                            videos\.funkwhale\.audio|
                            watch\.44con\.com|
                            peertube\.gcaillaut\.fr|
                            peertube\.icu|
                            pony\.tube|
                            spacepub\.space|
                            tube\.stbr\.io|
                            v\.mom-gay\.faith|
                            tube\.port0\.xyz|
                            peertube\.simounet\.net|
                            play\.jergefelt\.se|
                            peertube\.zeteo\.me|
                            tube\.danq\.me|
                            peertube\.kerenon\.com|
                            tube\.fab-l3\.org|
                            tube\.calculate\.social|
                            peertube\.mckillop\.org|
                            tube\.netzspielplatz\.de|
                            vod\.ksite\.de|
                            peertube\.laas\.fr|
                            tube\.govital\.net|
                            peertube\.stephenson\.cc|
                            bistule\.nohost\.me|
                            peertube\.kajalinifi\.de|
                            video\.ploud\.jp|
                            video\.omniatv\.com|
                            peertube\.ffs2play\.fr|
                            peertube\.leboulaire\.ovh|
                            peertube\.tronic-studio\.com|
                            peertube\.public\.cat|
                            peertube\.metalbanana\.net|
                            video\.1000i100\.fr|
                            peertube\.alter-nativ-voll\.de|
                            tube\.pasa\.tf|
                            tube\.worldofhauru\.xyz|
                            pt\.kamp\.site|
                            peertube\.teleassist\.fr|
                            videos\.mleduc\.xyz|
                            conf\.tube|
                            media\.privacyinternational\.org|
                            pt\.forty-two\.nl|
                            video\.halle-leaks\.de|
                            video\.grosskopfgames\.de|
                            peertube\.schaeferit\.de|
                            peertube\.jackbot\.fr|
                            tube\.extinctionrebellion\.fr|
                            peertube\.f-si\.org|
                            video\.subak\.ovh|
                            videos\.koweb\.fr|
                            peertube\.zergy\.net|
                            peertube\.roflcopter\.fr|
                            peertube\.floss-marketing-school\.com|
                            vloggers\.social|
                            peertube\.iriseden\.eu|
                            videos\.ubuntu-paris\.org|
                            peertube\.mastodon\.host|
                            armstube\.com|
                            peertube\.s2s\.video|
                            peertube\.lol|
                            tube\.open-plug\.eu|
                            open\.tube|
                            peertube\.ch|
                            peertube\.normandie-libre\.fr|
                            peertube\.slat\.org|
                            video\.lacaveatonton\.ovh|
                            peertube\.uno|
                            peertube\.servebeer\.com|
                            peertube\.fedi\.quebec|
                            tube\.h3z\.jp|
                            tube\.plus200\.com|
                            peertube\.eric\.ovh|
                            tube\.metadocs\.cc|
                            tube\.unmondemeilleur\.eu|
                            gouttedeau\.space|
                            video\.antirep\.net|
                            nrop\.cant\.at|
                            tube\.ksl-bmx\.de|
                            tube\.plaf\.fr|
                            tube\.tchncs\.de|
                            video\.devinberg\.com|
                            hitchtube\.fr|
                            peertube\.kosebamse\.com|
                            yunopeertube\.myddns\.me|
                            peertube\.varney\.fr|
                            peertube\.anon-kenkai\.com|
                            tube\.maiti\.info|
                            tubee\.fr|
                            videos\.dinofly\.com|
                            toobnix\.org|
                            videotape\.me|
                            voca\.tube|
                            video\.heromuster\.com|
                            video\.lemediatv\.fr|
                            video\.up\.edu\.ph|
                            balafon\.video|
                            video\.ivel\.fr|
                            thickrips\.cloud|
                            pt\.laurentkruger\.fr|
                            video\.monarch-pass\.net|
                            peertube\.artica\.center|
                            video\.alternanet\.fr|
                            indymotion\.fr|
                            fanvid\.stopthatimp\.net|
                            video\.farci\.org|
                            v\.lesterpig\.com|
                            video\.okaris\.de|
                            tube\.pawelko\.net|
                            peertube\.mablr\.org|
                            tube\.fede\.re|
                            pytu\.be|
                            evertron\.tv|
                            devtube\.dev-wiki\.de|
                            raptube\.antipub\.org|
                            video\.selea\.se|
                            peertube\.mygaia\.org|
                            video\.oh14\.de|
                            peertube\.livingutopia\.org|
                            peertube\.the-penguin\.de|
                            tube\.thechangebook\.org|
                            tube\.anjara\.eu|
                            pt\.pube\.tk|
                            video\.samedi\.pm|
                            mplayer\.demouliere\.eu|
                            widemus\.de|
                            peertube\.me|
                            peertube\.zapashcanon\.fr|
                            video\.latavernedejohnjohn\.fr|
                            peertube\.pcservice46\.fr|
                            peertube\.mazzonetto\.eu|
                            video\.irem\.univ-paris-diderot\.fr|
                            video\.livecchi\.cloud|
                            alttube\.fr|
                            video\.coop\.tools|
                            video\.cabane-libre\.org|
                            peertube\.openstreetmap\.fr|
                            videos\.alolise\.org|
                            irrsinn\.video|
                            video\.antopie\.org|
                            scitech\.video|
                            tube2\.nemsia\.org|
                            video\.amic37\.fr|
                            peertube\.freeforge\.eu|
                            video\.arbitrarion\.com|
                            video\.datsemultimedia\.com|
                            stoptrackingus\.tv|
                            peertube\.ricostrongxxx\.com|
                            docker\.videos\.lecygnenoir\.info|
                            peertube\.togart\.de|
                            tube\.postblue\.info|
                            videos\.domainepublic\.net|
                            peertube\.cyber-tribal\.com|
                            video\.gresille\.org|
                            peertube\.dsmouse\.net|
                            cinema\.yunohost\.support|
                            tube\.theocevaer\.fr|
                            repro\.video|
                            tube\.4aem\.com|
                            quaziinc\.com|
                            peertube\.metawurst\.space|
                            videos\.wakapo\.com|
                            video\.ploud\.fr|
                            video\.freeradical\.zone|
                            tube\.valinor\.fr|
                            refuznik\.video|
                            pt\.kircheneuenburg\.de|
                            peertube\.asrun\.eu|
                            peertube\.lagob\.fr|
                            videos\.side-ways\.net|
                            91video\.online|
                            video\.valme\.io|
                            video\.taboulisme\.com|
                            videos-libr\.es|
                            tv\.mooh\.fr|
                            nuage\.acostey\.fr|
                            video\.monsieur-a\.fr|
                            peertube\.librelois\.fr|
                            videos\.pair2jeux\.tube|
                            videos\.pueseso\.club|
                            peer\.mathdacloud\.ovh|
                            media\.assassinate-you\.net|
                            vidcommons\.org|
                            ptube\.rousset\.nom\.fr|
                            tube\.cyano\.at|
                            videos\.squat\.net|
                            video\.iphodase\.fr|
                            peertube\.makotoworkshop\.org|
                            peertube\.serveur\.slv-valbonne\.fr|
                            vault\.mle\.party|
                            hostyour\.tv|
                            videos\.hack2g2\.fr|
                            libre\.tube|
                            pire\.artisanlogiciel\.net|
                            videos\.numerique-en-commun\.fr|
                            video\.netsyms\.com|
                            video\.die-partei\.social|
                            video\.writeas\.org|
                            peertube\.swarm\.solvingmaz\.es|
                            tube\.pericoloso\.ovh|
                            watching\.cypherpunk\.observer|
                            videos\.adhocmusic\.com|
                            tube\.rfc1149\.net|
                            peertube\.librelabucm\.org|
                            videos\.numericoop\.fr|
                            peertube\.koehn\.com|
                            peertube\.anarchmusicall\.net|
                            tube\.kampftoast\.de|
                            vid\.y-y\.li|
                            peertube\.xtenz\.xyz|
                            diode\.zone|
                            tube\.egf\.mn|
                            peertube\.nomagic\.uk|
                            visionon\.tv|
                            videos\.koumoul\.com|
                            video\.rastapuls\.com|
                            video\.mantlepro\.com|
                            video\.deadsuperhero\.com|
                            peertube\.musicstudio\.pro|
                            peertube\.we-keys\.fr|
                            artitube\.artifaille\.fr|
                            peertube\.ethernia\.net|
                            tube\.midov\.pl|
                            peertube\.fr|
                            watch\.snoot\.tube|
                            peertube\.donnadieu\.fr|
                            argos\.aquilenet\.fr|
                            tube\.nemsia\.org|
                            tube\.bruniau\.net|
                            videos\.darckoune\.moe|
                            tube\.traydent\.info|
                            dev\.videos\.lecygnenoir\.info|
                            peertube\.nayya\.org|
                            peertube\.live|
                            peertube\.mofgao\.space|
                            video\.lequerrec\.eu|
                            peertube\.amicale\.net|
                            aperi\.tube|
                            tube\.ac-lyon\.fr|
                            video\.lw1\.at|
                            www\.yiny\.org|
                            videos\.pofilo\.fr|
                            tube\.lou\.lt|
                            choob\.h\.etbus\.ch|
                            tube\.hoga\.fr|
                            peertube\.heberge\.fr|
                            video\.obermui\.de|
                            videos\.cloudfrancois\.fr|
                            betamax\.video|
                            video\.typica\.us|
                            tube\.piweb\.be|
                            video\.blender\.org|
                            peertube\.cat|
                            tube\.kdy\.ch|
                            pe\.ertu\.be|
                            peertube\.social|
                            videos\.lescommuns\.org|
                            tv\.datamol\.org|
                            videonaute\.fr|
                            dialup\.express|
                            peertube\.nogafa\.org|
                            megatube\.lilomoino\.fr|
                            peertube\.tamanoir\.foucry\.net|
                            peertube\.devosi\.org|
                            peertube\.1312\.media|
                            tube\.bootlicker\.party|
                            skeptikon\.fr|
                            video\.blueline\.mg|
                            tube\.homecomputing\.fr|
                            tube\.ouahpiti\.info|
                            video\.tedomum\.net|
                            video\.g3l\.org|
                            fontube\.fr|
                            peertube\.gaialabs\.ch|
                            tube\.kher\.nl|
                            peertube\.qtg\.fr|
                            video\.migennes\.net|
                            tube\.p2p\.legal|
                            troll\.tv|
                            videos\.iut-orsay\.fr|
                            peertube\.solidev\.net|
                            videos\.cemea\.org|
                            video\.passageenseine\.fr|
                            videos\.festivalparminous\.org|
                            peertube\.touhoppai\.moe|
                            sikke\.fi|
                            peer\.hostux\.social|
                            share\.tube|
                            peertube\.walkingmountains\.fr|
                            videos\.benpro\.fr|
                            peertube\.parleur\.net|
                            peertube\.heraut\.eu|
                            tube\.aquilenet\.fr|
                            peertube\.gegeweb\.eu|
                            framatube\.org|
                            thinkerview\.video|
                            tube\.conferences-gesticulees\.net|
                            peertube\.datagueule\.tv|
                            video\.lqdn\.fr|
                            tube\.mochi\.academy|
                            media\.zat\.im|
                            video\.colibris-outilslibres\.org|
                            tube\.svnet\.fr|
                            peertube\.video|
                            peertube2\.cpy\.re|
                            peertube3\.cpy\.re|
                            videos\.tcit\.fr|
                            peertube\.cpy\.re|
                            canard\.tube
                        )'''
    _UUID_RE = r'[\da-zA-Z]{22}|[\da-fA-F]{8}-[\da-fA-F]{4}-[\da-fA-F]{4}-[\da-fA-F]{4}-[\da-fA-F]{12}'
    _API_BASE = 'https://%s/api/v1/videos/%s/%s'
    _VALID_URL = rf'''(?x)
                    (?:
                        peertube:(?P<host>[^:]+):|
                        https?://(?P<host_2>{_INSTANCES_RE})/(?:videos/(?:watch|embed)|api/v\d/videos|w)/
                    )
                    (?P<id>{_UUID_RE})
                    '''
    _EMBED_REGEX = [rf'''(?x)<iframe[^>]+\bsrc=["\'](?P<url>(?:https?:)?//{_INSTANCES_RE}/videos/embed/{_UUID_RE})''']
    _TESTS = [{
        'url': 'https://framatube.org/videos/watch/9c9de5e8-0a1e-484a-b099-e80766180a6d',
        'md5': '8563064d245a4be5705bddb22bb00a28',
        'info_dict': {
            'id': '9c9de5e8-0a1e-484a-b099-e80766180a6d',
            'ext': 'mp4',
            'title': 'What is PeerTube?',
            'description': 'md5:3fefb8dde2b189186ce0719fda6f7b10',
            'thumbnail': r're:https?://framatube\.org/lazy-static/thumbnails/.+\.jpg',
            'timestamp': 1538391166,
            'upload_date': '20181001',
            'uploader': 'Framasoft',
            'uploader_id': '3',
            'uploader_url': 'https://framatube.org/accounts/framasoft',
            'channel': 'A propos de PeerTube',
            'channel_id': '2215',
            'channel_url': 'https://framatube.org/video-channels/joinpeertube',
            'language': 'en',
            'license': 'Attribution - Share Alike',
            'duration': 113,
            'view_count': int,
            'like_count': int,
            'dislike_count': int,
            'tags': 'count:2',
            'categories': ['Science & Technology'],
        },
        'expected_warnings': ['HTTP Error 400: Bad Request'],
        'params': {'skip_download': 'm3u8'},
    }, {
        'url': 'https://peertube2.cpy.re/w/122d093a-1ede-43bd-bd34-59d2931ffc5e',
        'info_dict': {
            'id': '122d093a-1ede-43bd-bd34-59d2931ffc5e',
            'ext': 'mp4',
            'title': 'E2E tests',
            'categories': ['Unknown'],
            'channel': 'Main chocobozzz channel',
            'channel_id': '5187',
            'channel_url': 'https://peertube2.cpy.re/video-channels/chocobozzz_channel',
            'description': 'md5:67daf92c833c41c95db874e18fcb2786',
            'dislike_count': int,
            'duration': 52,
            'license': 'Unknown',
            'like_count': int,
            'tags': [],
            'thumbnail': r're:https?://peertube2\.cpy\.re/lazy-static/thumbnails/.+\.jpg',
            'timestamp': 1589276219,
            'upload_date': '20200512',
            'uploader': 'chocobozzz',
            'uploader_id': '37855',
            'uploader_url': 'https://peertube2.cpy.re/accounts/chocobozzz',
            'view_count': int,
        },
    }, {
        'url': 'https://peertube2.cpy.re/w/3fbif9S3WmtTP8gGsC5HBd',
        'info_dict': {
            'id': '3fbif9S3WmtTP8gGsC5HBd',
            'ext': 'mp4',
            'title': 'E2E tests',
            'categories': ['Unknown'],
            'channel': 'Main chocobozzz channel',
            'channel_id': '5187',
            'channel_url': 'https://peertube2.cpy.re/video-channels/chocobozzz_channel',
            'description': 'md5:67daf92c833c41c95db874e18fcb2786',
            'dislike_count': int,
            'duration': 52,
            'license': 'Unknown',
            'like_count': int,
            'tags': [],
            'thumbnail': r're:https?://peertube2\.cpy\.re/lazy-static/thumbnails/.+\.jpg',
            'timestamp': 1589276219,
            'upload_date': '20200512',
            'uploader': 'chocobozzz',
            'uploader_id': '37855',
            'uploader_url': 'https://peertube2.cpy.re/accounts/chocobozzz',
            'view_count': int,
        },
    }, {
        'url': 'https://peertube2.cpy.re/api/v1/videos/3fbif9S3WmtTP8gGsC5HBd',
        'info_dict': {
            'id': '3fbif9S3WmtTP8gGsC5HBd',
            'ext': 'mp4',
            'title': 'E2E tests',
            'categories': ['Unknown'],
            'channel': 'Main chocobozzz channel',
            'channel_id': '5187',
            'channel_url': 'https://peertube2.cpy.re/video-channels/chocobozzz_channel',
            'description': 'md5:67daf92c833c41c95db874e18fcb2786',
            'dislike_count': int,
            'duration': 52,
            'license': 'Unknown',
            'like_count': int,
            'tags': [],
            'thumbnail': r're:https?://peertube2\.cpy\.re/lazy-static/thumbnails/.+\.jpg',
            'timestamp': 1589276219,
            'upload_date': '20200512',
            'uploader': 'chocobozzz',
            'uploader_id': '37855',
            'uploader_url': 'https://peertube2.cpy.re/accounts/chocobozzz',
            'view_count': int,
        },
    }, {
        # https://github.com/ytdl-org/youtube-dl/issues/26002
        'url': 'peertube:spacepub.space:d8943b2d-8280-497b-85ec-bc282ec2afdc',
        'info_dict': {
            'id': 'd8943b2d-8280-497b-85ec-bc282ec2afdc',
            'ext': 'mp4',
            'title': 'Dot matrix printer shell demo',
            'uploader_id': '3',
            'timestamp': 1587401293,
            'upload_date': '20200420',
            'uploader': 'Drew DeVault',
        },
        'skip': 'Invalid URL',
    }, {
        'url': 'https://peertube.debian.social/videos/watch/0b04f13d-1e18-4f1d-814e-4979aa7c9c44',
        'only_matching': True,
    }, {
        # nsfw
        'url': 'https://vod.ksite.de/videos/watch/9bb88cd3-9959-46d9-9ab9-33d2bb704c39',
        'only_matching': True,
    }, {
        'url': 'https://vod.ksite.de/videos/embed/fed67262-6edb-4d1c-833b-daa9085c71d7',
        'only_matching': True,
    }, {
        'url': 'https://peertube.tv/api/v1/videos/c1875674-97d0-4c94-a058-3f7e64c962e8',
        'only_matching': True,
    }, {
        'url': 'peertube:framatube.org:b37a5b9f-e6b5-415c-b700-04a5cd6ec205',
        'only_matching': True,
    }]
    _WEBPAGE_TESTS = [{
        'url': 'https://video.macver.org/w/6gvhZpUGQVd4SQ6oYDc9pC',
        'info_dict': {
            'id': '6gvhZpUGQVd4SQ6oYDc9pC',
            'ext': 'mp4',
            'title': 'Minecraft, but if you say a block, it gets deleted',
            'categories': ['Gaming'],
            'channel': 'Waffle Irons Gaming',
            'channel_id': '4',
            'channel_url': 'https://video.macver.org/video-channels/waffle_irons',
            'description': 'md5:eda8daf64b0dadd00cc248f28eef213c',
            'dislike_count': int,
            'duration': 1643,
            'license': 'Attribution - Non Commercial',
            'like_count': int,
            'tags': 'count:1',
            'thumbnail': r're:https?://video\.macver\.org/lazy-static/thumbnails/.+\.jpg',
            'timestamp': 1751142352,
            'upload_date': '20250628',
            'uploader': 'Bog',
            'uploader_id': '3',
            'uploader_url': 'https://video.macver.org/accounts/bog',
            'view_count': int,
        },
        'expected_warnings': ['HTTP Error 400: Bad Request', 'Ignoring subtitle tracks found in the HLS manifest'],
        'params': {'skip_download': 'm3u8'},
    }]

    @staticmethod
    def _extract_peertube_url(webpage, source_url):
        mobj = re.match(
            rf'https?://(?P<host>[^/]+)/(?:videos/(?:watch|embed)|w)/(?P<id>{PeerTubeIE._UUID_RE})', source_url)
        if mobj and any(p in webpage for p in (
                'meta property="og:platform" content="PeerTube"',
                '<title>PeerTube<',
                'There will be other non JS-based clients to access PeerTube',
                '>We are sorry but it seems that PeerTube is not compatible with your web browser.<')):
            return 'peertube:{}:{}'.format(*mobj.group('host', 'id'))

    @classmethod
    def _extract_embed_urls(cls, url, webpage):
        embeds = tuple(super()._extract_embed_urls(url, webpage))
        if embeds:
            return embeds

        peertube_url = cls._extract_peertube_url(webpage, url)
        if peertube_url:
            return [peertube_url]

    def _call_api(self, host, video_id, path, note=None, errnote=None, fatal=True):
        return self._download_json(
            self._API_BASE % (host, video_id, path), video_id,
            note=note, errnote=errnote, fatal=fatal)

    def _get_subtitles(self, host, video_id):
        captions = self._call_api(
            host, video_id, 'captions', note='Downloading captions JSON',
            fatal=False)
        if not isinstance(captions, dict):
            return
        data = captions.get('data')
        if not isinstance(data, list):
            return
        subtitles = {}
        for e in data:
            language_id = try_get(e, lambda x: x['language']['id'], str)
            caption_url = urljoin(f'https://{host}', e.get('captionPath'))
            if not caption_url:
                continue
            subtitles.setdefault(language_id or 'en', []).append({
                'url': caption_url,
            })
        return subtitles

    def _real_extract(self, url):
        mobj = self._match_valid_url(url)
        host = mobj.group('host') or mobj.group('host_2')
        video_id = mobj.group('id')

        video = self._call_api(
            host, video_id, '', note='Downloading video JSON')

        title = video['name']

        formats, is_live = [], False
        files = video.get('files') or []
        for playlist in (video.get('streamingPlaylists') or []):
            if not isinstance(playlist, dict):
                continue
            if playlist_url := url_or_none(playlist.get('playlistUrl')):
                is_live = True
                formats.extend(self._extract_m3u8_formats(
                    playlist_url, video_id, fatal=False, live=True))
            playlist_files = playlist.get('files')
            if not (playlist_files and isinstance(playlist_files, list)):
                continue
            files.extend(playlist_files)
        for file_ in files:
            if not isinstance(file_, dict):
                continue
            file_url = url_or_none(file_.get('fileUrl'))
            if not file_url:
                continue
            file_size = int_or_none(file_.get('size'))
            format_id = try_get(
                file_, lambda x: x['resolution']['label'], str)
            f = parse_resolution(format_id)
            f.update({
                'url': file_url,
                'format_id': format_id,
                'filesize': file_size,
            })
            if format_id == '0p':
                f['vcodec'] = 'none'
            else:
                f['fps'] = int_or_none(file_.get('fps'))
            is_live = False
            formats.append(f)

        description = video.get('description')
        if description and len(description) >= 250:
            # description is shortened
            full_description = self._call_api(
                host, video_id, 'description', note='Downloading description JSON',
                fatal=False)

            if isinstance(full_description, dict):
                description = str_or_none(full_description.get('description')) or description

        subtitles = self.extract_subtitles(host, video_id)

        def data(section, field, type_):
            return try_get(video, lambda x: x[section][field], type_)

        def account_data(field, type_):
            return data('account', field, type_)

        def channel_data(field, type_):
            return data('channel', field, type_)

        category = data('category', 'label', str)
        categories = [category] if category else None

        nsfw = video.get('nsfw')
        if nsfw is bool:
            age_limit = 18 if nsfw else 0
        else:
            age_limit = None

        webpage_url = f'https://{host}/videos/watch/{video_id}'

        return {
            'id': video_id,
            'title': title,
            'description': description,
            'thumbnail': urljoin(webpage_url, video.get('thumbnailPath')),
            'timestamp': unified_timestamp(video.get('publishedAt')),
            'uploader': account_data('displayName', str),
            'uploader_id': str_or_none(account_data('id', int)),
            'uploader_url': url_or_none(account_data('url', str)),
            'channel': channel_data('displayName', str),
            'channel_id': str_or_none(channel_data('id', int)),
            'channel_url': url_or_none(channel_data('url', str)),
            'language': data('language', 'id', str),
            'license': data('licence', 'label', str),
            'duration': int_or_none(video.get('duration')),
            'view_count': int_or_none(video.get('views')),
            'like_count': int_or_none(video.get('likes')),
            'dislike_count': int_or_none(video.get('dislikes')),
            'age_limit': age_limit,
            'tags': try_get(video, lambda x: x['tags'], list),
            'categories': categories,
            'formats': formats,
            'subtitles': subtitles,
            'is_live': is_live,
            'webpage_url': webpage_url,
        }


class PeerTubePlaylistIE(InfoExtractor):
    IE_NAME = 'PeerTube:Playlist'
    _TYPES = {
        'a': 'accounts',
        'c': 'video-channels',
        'w/p': 'video-playlists',
    }
    _VALID_URL = r'''(?x)
                        https?://(?P<host>{})/(?P<type>(?:{}))/
                    (?P<id>[^/]+)
                    '''.format(PeerTubeIE._INSTANCES_RE, '|'.join(_TYPES.keys()))
    _TESTS = [{
        'url': 'https://peertube.debian.social/w/p/hFdJoTuyhNJVa1cDWd1d12',
        'info_dict': {
            'id': 'hFdJoTuyhNJVa1cDWd1d12',
            'description': 'Diversas palestras do Richard Stallman no Brasil.',
            'title': 'Richard Stallman no Brasil',
            'channel': 'debianbrazilteam',
            'channel_id': 1522,
            'thumbnail': r're:https?://peertube\.debian\.social/lazy-static/thumbnails/.+\.jpg',
            'timestamp': 1599676222,
            'upload_date': '20200909',
        },
        'playlist_mincount': 9,
    }, {
        'url': 'https://peertube2.cpy.re/a/chocobozzz/videos',
        'info_dict': {
            'id': 'chocobozzz',
            'title': 'chocobozzz',
            'channel': 'chocobozzz',
            'channel_id': 37855,
            'thumbnail': '',
            'timestamp': 1553874564,
            'upload_date': '20190329',
        },
        'playlist_mincount': 2,
    }, {
        'url': 'https://framatube.org/c/bf54d359-cfad-4935-9d45-9d6be93f63e8/videos',
        'info_dict': {
            'id': 'bf54d359-cfad-4935-9d45-9d6be93f63e8',
            'title': 'Les vidéos de Framasoft',
            'channel': 'framasoft',
            'channel_id': 3,
            'thumbnail': '',
            'timestamp': 1519917377,
            'upload_date': '20180301',
        },
        'playlist_mincount': 345,
    }, {
        'url': 'https://peertube2.cpy.re/c/blender_open_movies@video.blender.org/videos',
        'info_dict': {
            'id': 'blender_open_movies@video.blender.org',
            'title': 'Official Blender Open Movies',
            'channel': 'blender',
            'channel_id': 1926,
            'thumbnail': '',
            'timestamp': 1540472902,
            'upload_date': '20181025',
        },
        'playlist_mincount': 11,
    }]
    _API_BASE = 'https://%s/api/v1/%s/%s%s'
    _PAGE_SIZE = 30

    def call_api(self, host, name, path, base, **kwargs):
        return self._download_json(
            self._API_BASE % (host, base, name, path), name, **kwargs)

    def fetch_page(self, host, playlist_id, playlist_type, page):
        page += 1
        video_data = self.call_api(
            host, playlist_id,
            f'/videos?sort=-createdAt&start={self._PAGE_SIZE * (page - 1)}&count={self._PAGE_SIZE}&nsfw=both',
            playlist_type, note=f'Downloading page {page}').get('data', [])
        for video in video_data:
            short_uuid = video.get('shortUUID') or try_get(video, lambda x: x['video']['shortUUID'])
            video_title = video.get('name') or try_get(video, lambda x: x['video']['name'])
            yield self.url_result(
                f'https://{host}/w/{short_uuid}', PeerTubeIE.ie_key(),
                video_id=short_uuid, video_title=video_title)

    def _extract_playlist(self, host, playlist_type, playlist_id):
        info = self.call_api(host, playlist_id, '', playlist_type, note='Downloading playlist information', fatal=False)

        playlist_title = info.get('displayName')
        playlist_description = info.get('description')
        playlist_timestamp = unified_timestamp(info.get('createdAt'))
        channel = try_get(info, lambda x: x['ownerAccount']['name']) or info.get('displayName')
        channel_id = try_get(info, lambda x: x['ownerAccount']['id']) or info.get('id')
        thumbnail = format_field(info, 'thumbnailPath', f'https://{host}%s')

        entries = OnDemandPagedList(functools.partial(
            self.fetch_page, host, playlist_id, playlist_type), self._PAGE_SIZE)

        return self.playlist_result(
            entries, playlist_id, playlist_title, playlist_description,
            timestamp=playlist_timestamp, channel=channel, channel_id=channel_id, thumbnail=thumbnail)

    def _real_extract(self, url):
        playlist_type, host, playlist_id = self._match_valid_url(url).group('type', 'host', 'id')
        return self._extract_playlist(host, self._TYPES[playlist_type], playlist_id)



import urllib.parse

from .common import InfoExtractor
from ..utils import (
    clean_html,
    determine_ext,
    extract_attributes,
    get_element_by_class,
    get_element_html_by_class,
    int_or_none,
    url_or_none,
)
from ..utils.traversal import traverse_obj


class TV5MondePlusIE(InfoExtractor):
    IE_NAME = 'TV5MONDE'
    _VALID_URL = r'https?://(?:www\.)?tv5monde\.com/tv/video/(?P<id>[^/?#]+)'
    _TESTS = [{
        # documentary
        'url': 'https://www.tv5monde.com/tv/video/65931-baudouin-l-heritage-d-un-roi-baudouin-l-heritage-d-un-roi',
        'md5': 'd2a708902d3df230a357c99701aece05',
        'info_dict': {
            'id': '3FPa7JMu21_6D4BA7b',
            'display_id': '65931-baudouin-l-heritage-d-un-roi-baudouin-l-heritage-d-un-roi',
            'ext': 'mp4',
            'title': "Baudouin, l'héritage d'un roi",
            'thumbnail': 'https://psi.tv5monde.com/upsilon-images/960x540/6f/baudouin-f49c6b0e.jpg',
            'duration': 4842,
            'upload_date': '20240130',
            'timestamp': 1706641242,
            'episode': "BAUDOUIN, L'HERITAGE D'UN ROI",
            'description': 'md5:78125c74a5cac06d7743a2d09126edad',
            'series': "Baudouin, l'héritage d'un roi",
        },
    }, {
        # series episode
        'url': 'https://www.tv5monde.com/tv/video/52952-toute-la-vie-mardi-23-mars-2021',
        'md5': 'f5e09637cadd55639c05874e22eb56bf',
        'info_dict': {
            'id': 'obRRZ8m6g9_6D4BA7b',
            'display_id': '52952-toute-la-vie-mardi-23-mars-2021',
            'ext': 'mp4',
            'title': 'Toute la vie',
            'description': 'md5:a824a2e1dfd94cf45fa379a1fb43ce65',
            'thumbnail': 'https://psi.tv5monde.com/media/image/960px/5880553.jpg',
            'duration': 2526,
            'upload_date': '20230721',
            'timestamp': 1689971646,
            'series': 'Toute la vie',
            'episode': 'Mardi 23 mars 2021',
        },
    }, {
        # movie
        'url': 'https://www.tv5monde.com/tv/video/8771-ce-fleuve-qui-nous-charrie-ce-fleuve-qui-nous-charrie-p001-ce-fleuve-qui-nous-charrie',
        'md5': '87cefc34e10a6bf4f7823cccd7b36eb2',
        'info_dict': {
            'id': 'DOcfvdLKXL_6D4BA7b',
            'display_id': '8771-ce-fleuve-qui-nous-charrie-ce-fleuve-qui-nous-charrie-p001-ce-fleuve-qui-nous-charrie',
            'ext': 'mp4',
            'title': 'Ce fleuve qui nous charrie',
            'description': 'md5:62ba3f875343c7fc4082bdfbbc1be992',
            'thumbnail': 'https://psi.tv5monde.com/media/image/960px/5476617.jpg',
            'duration': 5300,
            'upload_date': '20210822',
            'timestamp': 1629594105,
            'episode': 'CE FLEUVE QUI NOUS CHARRIE-P001-CE FLEUVE QUI NOUS CHARRIE',
            'series': 'Ce fleuve qui nous charrie',
        },
    }, {
        # news
        'url': 'https://www.tv5monde.com/tv/video/70402-tv5monde-le-journal-edition-du-08-05-24-11h',
        'md5': 'c62977d6d10754a2ecebba70ad370479',
        'info_dict': {
            'id': 'LgQFrOCNsc_6D4BA7b',
            'display_id': '70402-tv5monde-le-journal-edition-du-08-05-24-11h',
            'ext': 'mp4',
            'title': 'TV5MONDE, le journal',
            'description': 'md5:777dc209eaa4423b678477c36b0b04a8',
            'thumbnail': 'https://psi.tv5monde.com/media/image/960px/6184105.jpg',
            'duration': 854,
            'upload_date': '20240508',
            'timestamp': 1715159640,
            'series': 'TV5MONDE, le journal',
            'episode': 'EDITION DU 08/05/24 - 11H',
        },
    }]
    _GEO_BYPASS = False

    @staticmethod
    def _extract_subtitles(data_captions):
        subtitles = {}
        for f in traverse_obj(data_captions, ('files', lambda _, v: url_or_none(v['file']))):
            subtitles.setdefault(f.get('label') or 'fra', []).append({'url': f['file']})
        return subtitles

    def _real_extract(self, url):
        display_id = self._match_id(url)
        webpage = self._download_webpage(url, display_id, impersonate=True)

        if ">Ce programme n'est malheureusement pas disponible pour votre zone géographique.<" in webpage:
            self.raise_geo_restricted(countries=['FR'])

        vpl_data = extract_attributes(self._search_regex(
            r'(<[^>]+class="video_player_loader"[^>]+>)',
            webpage, 'video player loader'))

        video_files = self._parse_json(
            vpl_data['data-broadcast'], display_id)
        formats = []
        video_id = None

        def process_video_files(v):
            nonlocal video_id
            for video_file in v:
                v_url = video_file.get('url')
                if not v_url:
                    continue
                if video_file.get('type') == 'application/deferred':
                    d_param = urllib.parse.quote(v_url)
                    token = video_file.get('token')
                    if not token:
                        continue
                    deferred_json = self._download_json(
                        f'https://api.tv5monde.com/player/asset/{d_param}/resolve?condenseKS=true',
                        display_id, 'Downloading deferred info', fatal=False, impersonate=True,
                        headers={'Authorization': f'Bearer {token}'})
                    v_url = traverse_obj(deferred_json, (0, 'url', {url_or_none}))
                    if not v_url:
                        continue
                    # data-guid from the webpage isn't stable, use the material id from the json urls
                    video_id = self._search_regex(
                        r'materials/([\da-zA-Z]{10}_[\da-fA-F]{7})/', v_url, 'video id', default=None)
                    process_video_files(deferred_json)

                video_format = video_file.get('format') or determine_ext(v_url)
                if video_format == 'm3u8':
                    formats.extend(self._extract_m3u8_formats(
                        v_url, display_id, 'mp4', 'm3u8_native',
                        m3u8_id='hls', fatal=False))
                elif video_format == 'mpd':
                    formats.extend(self._extract_mpd_formats(
                        v_url, display_id, fatal=False))
                else:
                    formats.append({
                        'url': v_url,
                        'format_id': video_format,
                    })

        process_video_files(video_files)

        metadata = self._parse_json(
            vpl_data.get('data-metadata') or '{}', display_id, fatal=False)

        if not video_id:
            video_id = self._search_regex(
                (r'data-guid=["\']([\da-f]{8}-[\da-f]{4}-[\da-f]{4}-[\da-f]{4}-[\da-f]{12})',
                 r'id_contenu["\']\s:\s*(\d+)'), webpage, 'video id',
                default=display_id)

        return {
            **traverse_obj(metadata, ('content', {
                'id': ('id', {str}),
                'title': ('title', {str}),
                'episode': ('title', {str}),
                'series': ('series', {str}),
                'timestamp': ('publishDate_ts', {int_or_none}),
                'duration': ('duration', {int_or_none}),
            })),
            'id': video_id,
            'display_id': display_id,
            'title': clean_html(get_element_by_class('main-title', webpage)),
            'description': clean_html(get_element_by_class('text', get_element_html_by_class('ep-summary', webpage) or '')),
            'thumbnail': url_or_none(vpl_data.get('data-image')),
            'formats': formats,
            'subtitles': self._extract_subtitles(self._parse_json(
                traverse_obj(vpl_data, ('data-captions', {str}), default='{}'), display_id, fatal=False)),
        }
